@@ -1,3 +1,21 @@
+!
+!     CalculiX - A 3-dimensional finite element program
+!              Copyright (C) 1998-2025 Guido Dhondt
+!
+!     This program is free software; you can redistribute it and/or
+!     modify it under the terms of the GNU General Public License as
+!     published by the Free Software Foundation(version 2);
+!     
+!
+!     This program is distributed in the hope that it will be useful,
+!     but WITHOUT ANY WARRANTY; without even the implied warranty of 
+!     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the 
+!     GNU General Public License for more details.
+!
+!     You should have received a copy of the GNU General Public License
+!     along with this program; if not, write to the Free Software
+!     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+!
       subroutine anisonl(w,vo,elas,s,ii1,jj1,weight)
 !
 !     This routine replaces the following lines in e_c3d.f for
@@ -27,8 +45,11 @@
 !                          enddo
 !                        enddo
 !                      enddo
+!
+      implicit none
 !
       integer ii1,jj1
+!
       real*8 w(3,3),vo(3,3),elas(21),s(60,60),weight
 !
       s(ii1,jj1)=s(ii1,jj1)+((elas( 1)+elas( 1)*vo(1,1)
