@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -29,6 +29,8 @@
 !     to speed up the calculation.
 !
       implicit none
+!
+      logical nodeslavsurf
 !
       character*6 prlab(*)
       character*8 lakon(*)
@@ -37,11 +39,11 @@
       character*87 filab(*)
 !
       integer imddof(*),nmddof,nrset,istartset(*),iendset(*),mi(*),
-     &  ialset(*),nactdof(0:mi(2),*),node,ithermal,j,k,l,
-     &  ikmpc(*),ilmpc(*),ipompc(*),nodempc(3,*),nmpc,
-     &  imdnode(*),nmdnode,imdmpc(*),nmdmpc,nprint,
-     &  imdboun(*),nmdboun,ikboun(*),nboun,index,indexe,islav,
-     &  jface,nset,ntie,nnodelem,nope,nodef(8),nelem,nface,iright,
+     &  ialset(*),nactdof(0:mi(2),*),node,ithermal(*),j,k,l,
+     &  ikmpc(*),ilmpc(*),ipompc(*),nodempc(3,*),nmpc,id,
+     &  imdnode(*),nmdnode,imdmpc(*),nmdmpc,nprint,ipos,
+     &  imdboun(*),nmdboun,ikboun(*),nboun,indexe1,indexe,islav,
+     &  jface,nset,ntie,nnodelem,nope,nodef(8),nelem,nface,imast,
      &  ifaceq(8,6),ifacet(6,4),ifacew1(4,5),ifacew2(8,5),kon(*),
      &  ipkon(*),i,ilboun(*),nlabel,ne,cyclicsymmetry
 !
@@ -77,7 +79,7 @@
      &             2,3,6,5,8,15,11,14,
      &             3,1,4,6,9,13,12,15/
 !
-      data nlabel /32/
+      data nlabel /56/
 !
 !     if 1d/2d elements are part of the mesh, no node selection
 !     is performed (because of the renumbering due to the
@@ -86,7 +88,7 @@
       do i=1,ne
          if((lakon(i)(7:7).eq.'E').or.
      &      (lakon(i)(7:7).eq.'S').or.
-     &      (lakon(i)(7:7).eq.'A').or.
+     &      ((lakon(i)(7:7).eq.'A').and.(lakon(i)(1:1).eq.'C')).or.
      &      (lakon(i)(7:7).eq.'L').or.
      &      (lakon(i)(7:7).eq.'B')) then
             nmdnode=0
@@ -97,7 +99,8 @@
          endif
       enddo
 !
-!     storing the nodes for which *NODE FILE or *EL FILE was selected
+!     storing the nodes, dofs, spcs and mpcs for which *NODE FILE 
+!     or *EL FILE was selected
 !
       do i=1,nlabel
 !
@@ -109,13 +112,20 @@
 !
          if(filab(i)(1:1).ne.' ') then
             read(filab(i)(7:87),'(a81)') noset
+c            nrset=0
+c            do k=1,nset
+c               if(set(k).eq.noset) then
+c                  nrset=k
+c                  exit
+c               endif
+c            enddo
+            call cident81(set,noset,nset,id)
             nrset=0
-            do k=1,nset
-               if(set(k).eq.noset) then
-                  nrset=k
-                  exit
-               endif
-            enddo
+            if(id.gt.0) then
+              if(noset.eq.set(id)) then
+                nrset=id
+              endif
+            endif
 !
 !           if output for all nodes is selected, use
 !           of imdnode is deactivated
@@ -127,8 +137,15 @@
                   write(*,*) '       steady static dynamics calculation'
                   write(*,*) '       a node set MUST be defined on each'
                   write(*,*) '       *NODE FILE, *NODE OUTPUT, *EL FILE'
-                  write(*,*) '       or *ELEMENT OUTPUT card'
-                  stop
+                  write(*,*) '       or *ELEMENT OUTPUT card.'
+                  write(*,*) '       Justification: in a steady state'
+                  write(*,*) '       dynamics calculation with cyclic'
+                  write(*,*) '       symmetry the segment is expanded'
+                  write(*,*) '       into 360 degrees. Storing results'
+                  write(*,*) '       for this expansion may lead to'
+                  write(*,*) '       huge frd-files. Specifying a set'
+                  write(*,*) '       can reduce this output.'
+                  call exit(201)
                endif
                nmdnode=0
                nmddof=0
@@ -143,7 +160,7 @@
                if(ialset(j).gt.0) then
                   node=ialset(j)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                   nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -163,7 +180,7 @@
                      node=node-ialset(j)
                      if(node.ge.ialset(j-1)) exit
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,
      &                      ipompc,nodempc,nmpc,imdnode,nmdnode,imddof,
@@ -184,7 +201,8 @@
          endif
       enddo
 !
-!     storing the nodes for which *NODE PRINT was selected
+!     storing the nodes, dofs, spcs and mpcs for which *NODE PRINT 
+!     was selected
 !
       do i=1,nprint
          if((prlab(i)(1:4).eq.'U   ').or.
@@ -196,13 +214,20 @@
      &        (prlab(i)(1:4).eq.'MF  ').or.
      &        (prlab(i)(1:4).eq.'V   ')) then
             noset=prset(i)
+c            nrset=0
+c            do k=1,nset
+c               if(set(k).eq.noset) then
+c                  nrset=k
+c                  exit
+c               endif
+c            enddo
+            call cident81(set,noset,nset,id)
             nrset=0
-            do k=1,nset
-               if(set(k).eq.noset) then
-                  nrset=k
-                  exit
-               endif
-            enddo
+            if(id.gt.0) then
+              if(noset.eq.set(id)) then
+                nrset=id
+              endif
+            endif
 !
 !           adding the nodes belonging to nrset
 !
@@ -210,7 +235,7 @@
                if(ialset(j).gt.0) then
                   node=ialset(j)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                   nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -230,7 +255,7 @@
                      node=node-ialset(j)
                      if(node.ge.ialset(j-1)) exit
                      call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
+                     if(ithermal(1).ne.2) then
                         do k=1,3
                            call addimdnodedof(node,k,ikmpc,ilmpc,
      &                      ipompc,nodempc,nmpc,imdnode,nmdnode,imddof,
@@ -250,7 +275,8 @@
          endif
       enddo
 !
-!     check whether all contact slave and master nodes were selected
+!     check whether all contact slave and master nodes (and corresponding
+!     dofs, spcs and mpcs were selected
 !
       do i=1,ntie
 !     
@@ -264,18 +290,25 @@
 !     
 !     determining the master surface
 !     
-            do j=1,nset
-               if(set(j).eq.rightset) exit
-            enddo
+c            do j=1,nset
+c               if(set(j).eq.rightset) exit
+c            enddo
+            call cident81(set,rightset,nset,id)
+            j=nset+1
+            if(id.gt.0) then
+              if(rightset.eq.set(id)) then
+                j=id
+              endif
+            endif
             if(j.gt.nset) then
                write(*,*) '*ERROR in createmddof: master surface',
      &              rightset
                write(*,*) '       does not exist'
-               stop
+               call exit(201)
             endif
-            iright=j
+            imast=j
 !     
-            do j=istartset(iright),iendset(iright)
+            do j=istartset(imast),iendset(imast)
 !     
                nelem=int(ialset(j)/10.d0)
                jface=ialset(j)-10*nelem
@@ -339,7 +372,7 @@
                do l=1,nnodelem
                   node=nodef(l)
                   call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
+                  if(ithermal(1).ne.2) then
                      do k=1,3
                         call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -354,65 +387,178 @@
 !
             slavset=tieset(2,i)
 !
-!           determining the slave surface
+!           check whether facial slave surface; 
 !
-            do j=1,nset
-               if(set(j).eq.slavset) exit
-            enddo
-            if(j.gt.nset) then
-               write(*,*) '*ERROR in createmddof: ',
-     &           'slave nodal surface ',slavset
-               write(*,*) '       does not exist'
-               stop
+            ipos=index(slavset,' ')-1
+! 
+!           default for node-to-surface contact is
+!           a nodal slave surface
+!
+            if(slavset(ipos:ipos).eq.'S') then
+!
+!              'S' means node-to-face contact, it does not mean
+!              that the slave surface is node-based;
+!              start with assuming a nodal surface (default),
+!              if not present, check for a facial surface
+!
+               nodeslavsurf=.true.
+            else
+               nodeslavsurf=.false.
             endif
+!
+!           determining the slave surface 
+!
+c            do j=1,nset
+c               if(set(j).eq.slavset) exit
+c            enddo
+            call cident81(set,slavset,nset,id)
+            j=nset+1
+            if(id.gt.0) then
+              if(slavset.eq.set(id)) then
+                j=id
+              endif
+            endif
+            if(j.gt.nset) then
+               do j=1,nset
+                  if((set(j)(1:ipos-1).eq.slavset(1:ipos-1)).and.
+     &                 (set(j)(ipos:ipos).eq.'T')) then
+                     nodeslavsurf=.false.
+                     exit
+                  endif
+               enddo
+            endif
+!
             islav=j
 !
-            do j=istartset(islav),iendset(islav)
-               if(ialset(j).gt.0) then
-                  node=ialset(j)
-                  call addimd(imdnode,nmdnode,node)
-                  if(ithermal.ne.2) then
-                     do k=1,3
-                        call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
+            if(nodeslavsurf) then
+!
+!              nodal slave surface
+!
+               do j=istartset(islav),iendset(islav)
+                  if(ialset(j).gt.0) then
+                     node=ialset(j)
+                     call addimd(imdnode,nmdnode,node)
+                     if(ithermal(1).ne.2) then
+                        do k=1,3
+                           call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
      &                       nactdof,mi,imdmpc,nmdmpc,imdboun,nmdboun,
      &                       ikboun,nboun,ilboun)
-                     enddo
-                  endif
-               else
-                  k=ialset(j-2)
-                  do
-                     k=k-ialset(j)
-                     if(k.ge.ialset(j-1)) exit
-                     node=k
-                     call addimd(imdnode,nmdnode,node)
-                     if(ithermal.ne.2) then
-                        do k=1,3
-                           call addimdnodedof(node,k,ikmpc,ilmpc,
+                        enddo
+                     endif
+                  else
+                     k=ialset(j-2)
+                     do
+                        k=k-ialset(j)
+                        if(k.ge.ialset(j-1)) exit
+                        node=k
+                        call addimd(imdnode,nmdnode,node)
+                        if(ithermal(1).ne.2) then
+                           do k=1,3
+                              call addimdnodedof(node,k,ikmpc,ilmpc,
      &                          ipompc,nodempc,nmpc,imdnode,nmdnode,
      &                          imddof,nmddof,nactdof,mi,imdmpc,nmdmpc,
      &                          imdboun,nmdboun,ikboun,nboun,ilboun)
+                           enddo
+                        endif
+                     enddo
+                  endif
+               enddo
+            else
+!     
+!             facial slave surface
+!
+               do j=istartset(islav),iendset(islav)
+!     
+                  nelem=int(ialset(j)/10.d0)
+                  jface=ialset(j)-10*nelem
+!     
+                  indexe=ipkon(nelem)
+!     
+                  if(lakon(nelem)(4:4).eq.'2') then
+                     nnodelem=8
+                     nface=6
+                  elseif(lakon(nelem)(4:4).eq.'8') then
+                     nnodelem=4
+                     nface=6
+                  elseif(lakon(nelem)(4:5).eq.'10') then
+                     nnodelem=6
+                     nface=4
+                  elseif(lakon(nelem)(4:4).eq.'4') then
+                     nnodelem=3
+                     nface=4
+                  elseif(lakon(nelem)(4:5).eq.'15') then
+                     if(jface.le.2) then
+                        nnodelem=6
+                     else
+                        nnodelem=8
+                     endif
+                     nface=5
+                     nope=15
+                  elseif(lakon(nelem)(4:4).eq.'6') then
+                     if(jface.le.2) then
+                        nnodelem=3
+                     else
+                        nnodelem=4
+                     endif
+                     nface=5
+                     nope=6
+                  else
+                     cycle
+                  endif
+!     
+!     determining the slave nodes 
+!     
+                  if(nface.eq.4) then
+                     do k=1,nnodelem
+                        nodef(k)=kon(indexe+ifacet(k,jface))
+                     enddo
+                  elseif(nface.eq.5) then
+                     if(nope.eq.6) then
+                        do k=1,nnodelem
+                           nodef(k)=kon(indexe+ifacew1(k,jface))
+                        enddo
+                     elseif(nope.eq.15) then
+                        do k=1,nnodelem
+                           nodef(k)=kon(indexe+ifacew2(k,jface))
+                        enddo
+                     endif
+                  elseif(nface.eq.6) then
+                     do k=1,nnodelem
+                        nodef(k)=kon(indexe+ifaceq(k,jface))
+                     enddo
+                  endif
+!     
+                  do l=1,nnodelem
+                     node=nodef(l)
+                     call addimd(imdnode,nmdnode,node)
+                     if(ithermal(1).ne.2) then
+                        do k=1,3
+                           call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
+     &                       nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
+     &                       nactdof,mi,imdmpc,nmdmpc,imdboun,nmdboun,
+     &                       ikboun,nboun,ilboun)
                         enddo
                      endif
                   enddo
-               endif
-            enddo
-!
+               enddo
+            endif
+!     
          endif
       enddo
 !
-!     adding nodes belonging to nonlinear MPC's (why only dependent nodes?)
+!     adding nodes, dofs, spcs and mpcs belonging to nonlinear MPC's 
+!     (why only dependent nodes?)
 !      
       do i=1,nmpc
          if((labmpc(i)(1:20).ne.'                    ').and.
-     &          (labmpc(i)(1:7).ne.'CONTACT').and.
      &          (labmpc(i)(1:6).ne.'CYCLIC').and.
      &          (labmpc(i)(1:9).ne.'SUBCYCLIC')) then
-            index=ipompc(i)
-            if(index.eq.0) cycle
-            node=nodempc(1,index)
+            indexe1=ipompc(i)
+            if(indexe1.eq.0) cycle
+            node=nodempc(1,indexe1)
             call addimd(imdnode,nmdnode,node)
-            if(ithermal.ne.2) then
+            if(ithermal(1).ne.2) then
                do k=1,3
                   call addimdnodedof(node,k,ikmpc,ilmpc,ipompc,
      &                 nodempc,nmpc,imdnode,nmdnode,imddof,nmddof,
@@ -428,14 +574,6 @@
             endif
          endif
       enddo
-!
-!     subtracting 1 to comply with the C-convention
-!
-c      do j=1,nmddof
-c         imddof(j)=imddof(j)-1
-c      enddo
-!
-c      write (*,*) 'nmddof, nmdnode',nmddof,nmdnode
 !
       return
       end
