@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -20,7 +20,7 @@
      &  ff,nelem,nmethod,rhcon,ielmat,ntmat_,vold,iperturb,nelemload,
      &  sideload,xload,nload,idist,ttime,time,istep,iinc,dtime,
      &  xloadold,reltime,ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,
-     &  veold,matname,mi)
+     &  veold,matname,mi,ielprop,prop)
 !
 !     computation of the rhs for the element with
 !     the topology in konl: only for nonlinear calculations (i.e.
@@ -40,8 +40,8 @@
       character*80 matname(*),amat
 !
       integer mi(*),nk,konl(20),ielmat(mi(3),*),nbody,ifaceq(8,6),
-     &  nelemload(2,*),
-     &  nelem,nmethod,iperturb,nload,idist,i,i1,j1,jj,jj1,id,kk,
+     &  nelemload(2,*),ielprop(*),null,
+     &  nelem,nmethod,iperturb(*),nload,idist,i,i1,j1,jj,jj1,id,kk,
      &  ipointer,nope,nopes,j,k,ntmat_,i2,imat,ii,ig,mint2d,mint3d,
      &  ifacet(6,4),ifacew(8,5),istep,iinc,layer,kspt,jltyp,iflag,
      &  ipompc(*),nodempc(3,*),nmpc,ikmpc(*),ilmpc(*),iscale
@@ -50,7 +50,7 @@
      &  rhcon(0:1,ntmat_,*),xs2(3,7),xloadold(2,*),reltime,
      &  bodyf(3),om(2),rho,bf(3),q(3),shpj(4,20),xl(3,20),
      &  shp(4,20),voldl(3,20),xl2(3,8),xsj2(3),shp2(7,8),
-     &  vold(0:mi(2),*),
+     &  vold(0:mi(2),*),prop(*),
      &  xload(2,*),xi,et,ze,const,xsj,ff(60),weight,ttime,time,tvar(2),
      &  coords(3),dtime,coefmpc(*)
 !
@@ -72,9 +72,10 @@
      &             2,3,6,5,8,15,11,14,
      &             4,6,3,1,12,15,9,13/
       data iflag /3/
+      data null /0/
 !
       tvar(1)=time
-      tvar(2)=ttime+dtime
+      tvar(2)=ttime+time
 !
       if(lakonl(4:4).eq.'2') then
          nope=20
@@ -97,6 +98,15 @@
       if(lakonl(4:5).eq.'8R') then
          mint2d=1
          mint3d=1
+      elseif(lakonl(4:7).eq.'20RB') then
+         if((lakonl(8:8).eq.'R').or.(lakonl(8:8).eq.'C')) then
+            mint2d=4
+            mint3d=50
+         else
+            mint2d=4
+            call beamintscheme(lakonl,mint3d,ielprop(nelem),prop,
+     &           null,xi,et,ze,weight)
+         endif
       elseif((lakonl(4:4).eq.'8').or.(lakonl(4:6).eq.'20R')) then
          mint2d=4
          mint3d=8
@@ -133,7 +143,8 @@ c      endif
 !
 !     displacements for 2nd order static and modal theory
 !
-      if(iperturb.ne.0) then
+c      if(iperturb.ne.0) then
+      if((iperturb(1).eq.1).or.(iperturb(2).eq.1)) then
          do i1=1,nope
             do i2=1,3
                voldl(i2,i1)=vold(i2,konl(i1))
@@ -184,6 +195,16 @@ c      endif
                et=gauss3d1(2,kk)
                ze=gauss3d1(3,kk)
                weight=weight3d1(kk)
+            elseif(lakonl(4:7).eq.'20RB') then
+               if((lakonl(8:8).eq.'R').or.(lakonl(8:8).eq.'C')) then
+                  xi=gauss3d13(1,kk)
+                  et=gauss3d13(2,kk)
+                  ze=gauss3d13(3,kk)
+                  weight=weight3d13(kk)
+               else
+                  call beamintscheme(lakonl,mint3d,ielprop(nelem),prop,
+     &                 kk,xi,et,ze,weight)
+               endif
             elseif((lakonl(4:4).eq.'8').or.(lakonl(4:6).eq.'20R')) 
      &              then
                xi=gauss3d2(1,kk)
@@ -328,7 +349,9 @@ c      endif
 !     point
 !     
                      q(i1)=0.d0
-                     if(iperturb.eq.0) then
+c                     if(iperturb.eq.0) then
+                     if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) 
+     &                    then
                         do j1=1,nope
                            q(i1)=q(i1)+shp(4,j1)*xl(i1,j1)
                         enddo
@@ -401,7 +424,8 @@ c      endif
          endif
 !     
          if((nope.eq.20).or.(nope.eq.8)) then
-            if(iperturb.eq.0) then
+c            if(iperturb.eq.0) then
+            if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                do i=1,nopes
                   do j=1,3
                      xl2(j,i)=co(j,konl(ifaceq(i,ig)))
@@ -416,7 +440,8 @@ c      endif
                enddo
             endif
          elseif((nope.eq.10).or.(nope.eq.4)) then
-            if(iperturb.eq.0) then
+c            if(iperturb.eq.0) then
+            if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                do i=1,nopes
                   do j=1,3
                      xl2(j,i)=co(j,konl(ifacet(i,ig)))
@@ -431,7 +456,8 @@ c      endif
                enddo
             endif
          else
-            if(iperturb.eq.0) then
+c            if(iperturb.eq.0) then
+            if((iperturb(1).ne.1).and.(iperturb(2).ne.1)) then
                do i=1,nopes
                   do j=1,3
                      xl2(j,i)=co(j,konl(ifacew(i,ig)))
