@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,7 +16,7 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine e_c3d_th(co,nk,konl,lakonl,s,sm,
+      subroutine e_c3d_th(co,nk,kon,lakonl,s,sm,
      &  ff,nelem,nmethod,rhcon,nrhcon,ielmat,ielorien,norien,orab,
      &  ntmat_,t0,t1,ithermal,vold,iperturb,nelemload,
      &  sideload,xload,nload,idist,iexpl,dtime,
@@ -24,7 +24,10 @@
      &  physcon,shcon,nshcon,cocon,ncocon,ttime,time,istep,iinc,
      &  xstiff,xloadold,reltime,
      &  ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,springarea,
-     &  plicon,nplicon,npmat_,ncmat_,elcon,nelcon)
+     &  plkcon,nplkcon,npmat_,ncmat_,elcon,nelcon,lakon,
+     &  pslavsurf,pmastsurf,mortar,clearini,plicon,nplicon,ipkon,
+     &  ielprop,prop,iponoeln,inoeln,sti,xstateini,xstate,nstate_,
+     &  network,ipobody,xbody,ibody)
 !
 !     computation of the element matrix and rhs for the element with
 !     the topology in konl
@@ -39,71 +42,79 @@
 !
       implicit none
 !
-      logical mass,stiffness,buckling,rhsi
+      integer mass,stiffness,buckling,rhsi
 !
-      character*8 lakonl
+      character*8 lakonl,lakon(*)
       character*20 sideload(*)
       character*80 matname(*),amat
 !
-      integer konl(20),ifaceq(8,6),nelemload(2,*),nk,nelem,nmethod,
-     &  mattyp,ithermal(2),iperturb,nload,idist,i,j,k,i1,j1,l,m,
+      integer konl(26),ifaceq(8,6),nelemload(2,*),nk,nelem,nmethod,
+     &  mattyp,ithermal(*),iperturb(*),nload,idist,i,j,k,i1,j1,l,m,
      &  ii,jj,id,ipointer,ig,kk,istiff,iperm(20),ipompc(*),mi(*),
      &  nrhcon(*),ielmat(mi(3),*),ielorien(mi(3),*),nodempc(3,*),nmpc,
      &  ntmat_,nope,nopes,norien,iexpl,imat,mint2d,ikmpc(*),
-     &  mint3d,ifacet(6,4),nopev,iorien,ilmpc(*),kode,
+     &  mint3d,ifacet(6,4),nopev,iorien,ilmpc(*),kode,jfaces,null,
      &  ifacew(8,5),intscheme,ipointeri,ipointerj,ncocon(2,*),
-     &  nshcon(*),iinc,istep,jltyp,nfield,node,iflag,iscale,
-     &  nplicon(0:ntmat_,*),nelcon(2,*),npmat_,ncmat_,i2,
-     &  iemchange
+     &  nshcon(*),iinc,istep,jltyp,nfield,node,iflag,iscale,ielprop(*),
+     &  nplkcon(0:ntmat_,*),nelcon(2,*),npmat_,ncmat_,i2,ipkon(*),
+     &  iemchange,kon(*),mortar,nplicon(0:ntmat_,*),indexe,igauss,
+     &  iponoeln(*),inoeln(2,*),nstate_,network,ipobody(2,*),ibody(3,*)
 !
       real*8 co(3,*),xl(3,20),shp(4,20),xstiff(27,mi(1),*),
      &  s(60,60),w(3,3),ff(60),shpj(4,20),sinktemp,xs2(3,7),
      &  rhcon(0:1,ntmat_,*),dxsj2,temp,press,xloadold(2,*),
-     &  orab(7,*),t0(*),t1(*),coords(3),c1,c2,reltime,
-     &  xl2(3,8),xsj2(3),shp2(7,8),vold(0:mi(2),*),xload(2,*),
+     &  orab(7,*),t0(*),t1(*),coords(3),c1,c2,reltime,prop(*),
+     &  xl2(3,9),xsj2(3),shp2(7,9),vold(0:mi(2),*),xload(2,*),
      &  xi,et,ze,xsj,xsjj,sm(60,60),t1l,rho,summass,summ,ttime,time,
      &  sume,factorm,factore,alp,weight,pgauss(3),timeend(2),
      &  cocon(0:6,ntmat_,*),shcon(0:3,ntmat_,*),sph,coconloc(6),
      &  field,areaj,sax(60,60),ffax(60),coefmpc(*),tl2(8),
-     &  voldl(0:mi(2),20),springarea(2,*),plicon(0:2*npmat_,ntmat_,*),
-     &  elcon(0:ncmat_,ntmat_,*),elconloc(21)
+     &  voldl(0:mi(2),20),springarea(2,*),plkcon(0:2*npmat_,ntmat_,*),
+     &  elcon(0:ncmat_,ntmat_,*),elconloc(ncmat_),pslavsurf(3,*),
+     &  pmastsurf(2,*),clearini(3,9,*),plicon(0:2*npmat_,ntmat_,*),
+     &  sti(6,mi(1),*),xstate(nstate_,mi(1),*),xbody(7,*),
+     &  xstateini(nstate_,mi(1),*),heatnod,heatfac
 !
       real*8 dtime,physcon(*)
 !
       include "gauss.f"
 !
-      data ifaceq /4,3,2,1,11,10,9,12,
+      ifaceq=reshape((/4,3,2,1,11,10,9,12,
      &            5,6,7,8,13,14,15,16,
      &            1,2,6,5,9,18,13,17,
      &            2,3,7,6,10,19,14,18,
      &            3,4,8,7,11,20,15,19,
-     &            4,1,5,8,12,17,16,20/
-      data ifacet /1,3,2,7,6,5,
+     &            4,1,5,8,12,17,16,20/),(/8,6/))
+      ifacet=reshape((/1,3,2,7,6,5,
      &             1,2,4,5,9,8,
      &             2,3,4,6,10,9,
-     &             1,4,3,8,10,7/
-      data ifacew /1,3,2,9,8,7,0,0,
+     &             1,4,3,8,10,7/),(/6,4/))
+      ifacew=reshape((/1,3,2,9,8,7,0,0,
      &             4,5,6,10,11,12,0,0,
      &             1,2,5,4,7,14,10,13,
      &             2,3,6,5,8,15,11,14,
-     &             4,6,3,1,12,15,9,13/
-      data iflag /3/
-      data iperm /5,6,7,8,1,2,3,4,13,14,15,16,9,10,11,12,17,18,19,20/
+     &             4,6,3,1,12,15,9,13/),(/8,5/))
+      null=0
+      iperm=(/5,6,7,8,1,2,3,4,13,14,15,16,9,10,11,12,17,18,19,20/)
+!
+      iflag=3
 !
       timeend(1)=time
-      timeend(2)=ttime+dtime
+      timeend(2)=ttime+time
 !
       summass=0.d0
+      heatfac=0.d0
 !
+      indexe=ipkon(nelem)
       imat=ielmat(1,nelem)
       amat=matname(imat)
       if(norien.gt.0) then
-         iorien=ielorien(1,nelem)
+         iorien=max(0,ielorien(1,nelem))
       else
          iorien=0
       endif
 !
-      if(lakonl(4:4).eq.'2') then
+      if(lakonl(4:5).eq.'20') then
          nope=20
          nopev=8
          nopes=8
@@ -126,26 +137,41 @@
          nope=6
          nopev=6
       elseif(lakonl(1:2).eq.'ES') then
-         read(lakonl(8:8),'(i1)') nope
+         if(lakonl(7:7).eq.'C') then
+            if(mortar.eq.0) then
+               nope=ichar(lakonl(8:8))-47
+               konl(nope+1)=kon(indexe+nope+1)
+            elseif(mortar.eq.1) then
+               nope=kon(indexe)
+            endif
+         else
+            nope=ichar(lakonl(8:8))-47
+         endif
+      elseif((lakonl(1:2).eq.'D ').or.
+     &       ((lakonl(1:1).eq.'D').and.(network.eq.1))) then
+         nope=3
       endif
 !
-      if(intscheme.eq.0) then
+c      if(intscheme.eq.0) then
+!
+!        # of 2D and 3D integration points
+!
          if(lakonl(4:5).eq.'8R') then
             mint2d=1
             mint3d=1
+         elseif(lakonl(4:7).eq.'20RB') then
+            if((lakonl(8:8).eq.'R').or.(lakonl(8:8).eq.'C')) then
+               mint2d=4
+               mint3d=50
+            else
+               mint2d=4
+               call beamintscheme(lakonl,mint3d,ielprop(nelem),prop,
+     &              null,xi,et,ze,weight)
+            endif
          elseif((lakonl(4:4).eq.'8').or.(lakonl(4:6).eq.'20R')) then
-c            if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'S').or.
-c     &         (lakonl(7:7).eq.'E')) then
             if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E')) then
                mint2d=2
                mint3d=4
-c            elseif(lakonl(7:7).eq.'S') then
-c!
-c!              special treatment for plane stress elements
-c!              since lateral film heating is allowed
-c!
-c               mint2d=4
-c               mint3d=4
             else
                mint2d=4
                mint3d=8
@@ -166,36 +192,11 @@ c               mint3d=4
          else
             mint3d=0
          endif
-      else
-         if((lakonl(4:4).eq.'8').or.(lakonl(4:4).eq.'2')) then
-            mint3d=27
-         elseif((lakonl(4:5).eq.'10').or.(lakonl(4:4).eq.'4')) then
-            mint3d=15
-         elseif((lakonl(4:5).eq.'15').or.(lakonl(4:4).eq.'6')) then
-            mint3d=18
-         else
-            mint3d=0
-         endif
-         if(lakonl(4:5).eq.'8R') then
-            mint2d=1
-         elseif((lakonl(4:4).eq.'8').or.(lakonl(4:6).eq.'20R')) then
-            if((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E')) then
-               mint2d=2
-            else
-               mint2d=4
-            endif
-         elseif(lakonl(4:4).eq.'2') then
-            mint2d=9
-         elseif(lakonl(4:5).eq.'10') then
-            mint2d=3
-         elseif(lakonl(4:4).eq.'4') then
-            mint2d=1
-         endif
-      endif
 !
 !     computation of the coordinates of the local nodes
 !
       do i=1,nope
+        konl(i)=kon(indexe+i)
         do j=1,3
           xl(j,i)=co(j,konl(i))
         enddo
@@ -203,7 +204,7 @@ c               mint3d=4
 !
 !       initialisation for distributed forces
 !
-      if(rhsi) then
+      if(rhsi.eq.1) then
         if(idist.ne.0) then
           do i=1,nope
             ff(i)=0.d0
@@ -213,9 +214,8 @@ c               mint3d=4
 !
 !     initialisation of sm
 !
-      if(mass.or.buckling) then
+      if((mass.eq.1).or.(buckling.eq.1)) then
         do i=1,nope
-c          do j=1,nope
           do j=i,nope
             sm(i,j)=0.d0
           enddo
@@ -234,29 +234,79 @@ c          do j=1,nope
 !
       if(mint3d.eq.0) then
          do i1=1,nope
-c            do i2=1,3
             do i2=0,3
                voldl(i2,i1)=vold(i2,konl(i1))
             enddo
          enddo
-         kode=nelcon(1,imat)
-         if(kode.eq.-51)
-     &      call springstiff_th(xl,voldl,s,imat,elcon,nelcon,
-     &        ncmat_,ntmat_,nope,kode,plicon,
-     &        nplicon,npmat_,iperturb,springarea(1,konl(nope+1)),mi,
-     &        timeend,matname,konl(nope),nelem,istep,iinc)
+!
+         if(lakonl(1:2).eq.'ES') then
+            if(lakonl(7:7).eq.'C') then
+!
+!              contact element
+!
+               kode=nelcon(1,imat)
+               if(kode.eq.-51) then
+                  if(mortar.eq.0) then
+                     call springstiff_n2f_th(xl,voldl,s,imat,elcon,
+     &                 nelcon,ncmat_,ntmat_,nope,kode,plkcon,nplkcon,
+     &                 npmat_,iperturb,springarea(1,konl(nope+1)),mi,
+     &                 timeend,matname,konl(nope),nelem,istep,iinc)
+                  elseif(mortar.eq.1) then
+                     jfaces=kon(indexe+nope+2)
+                     igauss=kon(indexe+nope+1) 
+                     node=0
+                     call springstiff_f2f_th(xl,voldl,s,imat,elcon,
+     &                    nelcon,ncmat_,ntmat_,nope,lakonl,kode,
+     &                    elconloc,plicon,nplicon,npmat_,
+     &                    springarea(1,igauss),
+     &                    nmethod,mi,reltime,jfaces,igauss,pslavsurf,
+     &                    pmastsurf,clearini,matname,plkcon,nplkcon,
+     &                    node,nelem,istep,iinc,timeend)
+                  endif
+               endif
+            elseif(lakonl(7:7).eq.'F') then
+!
+!              advective element
+!
+               call advecstiff(nope,voldl,ithermal,xl,nelemload,
+     &              nelem,nload,lakon,xload,istep,time,ttime,dtime,
+     &              sideload,vold,mi,xloadold,reltime,nmethod,s,
+     &              iinc,iponoeln,inoeln,ielprop,prop,ielmat,shcon,
+     &              nshcon,rhcon,nrhcon,ntmat_,ipkon,kon,cocon,ncocon,
+     &              ipobody,xbody,ibody)
+            endif
+         elseif((lakonl(1:2).eq.'D ').or.
+     &          ((lakonl(1:1).eq.'D').and.(network.eq.1))) then
+            do i=1,3
+               do j=1,i
+                  s(i,j)=0.d0
+               enddo
+            enddo
+            call networkstiff(voldl,s,imat,konl,mi,ntmat_,shcon,
+     &           nshcon,rhcon,nrhcon)
+         endif
          return
       endif
 !
 !     computation of the matrix: loop over the Gauss points
 !
       do kk=1,mint3d
-         if(intscheme.eq.0) then
+c         if(intscheme.eq.0) then
             if(lakonl(4:5).eq.'8R') then
                xi=gauss3d1(1,kk)
                et=gauss3d1(2,kk)
                ze=gauss3d1(3,kk)
                weight=weight3d1(kk)
+            elseif(lakonl(4:7).eq.'20RB') then
+               if((lakonl(8:8).eq.'R').or.(lakonl(8:8).eq.'C')) then
+                  xi=gauss3d13(1,kk)
+                  et=gauss3d13(2,kk)
+                  ze=gauss3d13(3,kk)
+                  weight=weight3d13(kk)
+               else
+                  call beamintscheme(lakonl,mint3d,ielprop(nelem),prop,
+     &                 kk,xi,et,ze,weight)
+               endif
             elseif((lakonl(4:4).eq.'8').or.(lakonl(4:6).eq.'20R')) 
      &              then
                xi=gauss3d2(1,kk)
@@ -289,24 +339,6 @@ c            do i2=1,3
                ze=gauss3d7(3,kk)
                weight=weight3d7(kk)
             endif
-         else
-            if((lakonl(4:4).eq.'8').or.(lakonl(4:4).eq.'2')) then
-               xi=gauss3d3(1,kk)
-               et=gauss3d3(2,kk)
-               ze=gauss3d3(3,kk)
-               weight=weight3d3(kk)
-            elseif((lakonl(4:5).eq.'10').or.(lakonl(4:4).eq.'4')) then
-               xi=gauss3d6(1,kk)
-               et=gauss3d6(2,kk)
-               ze=gauss3d6(3,kk)
-               weight=weight3d6(kk)
-            else
-               xi=gauss3d9(1,kk)
-               et=gauss3d9(2,kk)
-               ze=gauss3d9(3,kk)
-               weight=weight3d9(kk)
-            endif
-         endif
 !
 !           calculation of the shape functions and their derivatives
 !           in the gauss point
@@ -362,10 +394,6 @@ c            do i2=1,3
             enddo
          endif
 !
-c         do i1=1,nope
-c            t1l=t1l+shp(4,i1)*vold(0,konl(i1))
-c         enddo
-!
 !           calculating the coordinates of the integration point
 !           for material orientation purposes (for cylindrical
 !           coordinate systems)
@@ -374,7 +402,6 @@ c         enddo
             do j=1,3
                pgauss(j)=0.d0
                do i1=1,nope
-c                  pgauss(j)=pgauss(j)+shp(4,i1)*co(j,konl(i1))
                   pgauss(j)=pgauss(j)+shp(4,i1)*xl(j,i1)
                enddo
             enddo
@@ -408,15 +435,6 @@ c                  pgauss(j)=pgauss(j)+shp(4,i1)*co(j,konl(i1))
 !
             do ii=1,jj
 !
-!                   all products of the shape functions for a given ii
-!                   and jj
-!
-c               do i1=1,3
-c                  do j1=1,3
-c                     w(i1,j1)=shpj(i1,ii)*shpj(j1,jj)
-c                  enddo
-c               enddo
-!
 !                   the following section calculates the static
 !                   part of the stiffness matrix which, for buckling 
 !                   calculations, is done in a preliminary static
@@ -424,17 +442,12 @@ c               enddo
 !
                if(mattyp.eq.1) then
 !
-c                  s(ii,jj)=s(ii,jj)+coconloc(1)*
-c     &                 (w(1,1)+w(2,2)+w(3,3))*weight
                   s(ii,jj)=s(ii,jj)+c1*
      &                 (shpj(1,ii)*shpj(1,jj)+shpj(2,ii)*shpj(2,jj)
      &                 +shpj(3,ii)*shpj(3,jj))
 !
                elseif(mattyp.eq.2) then
 !
-c                  s(ii,jj)=s(ii,jj)+(coconloc(1)*w(1,1)+
-c     &                 coconloc(2)*w(2,2)+coconloc(3)*w(3,3))*
-c     &                 weight
                   s(ii,jj)=s(ii,jj)+(coconloc(1)*shpj(1,ii)*shpj(1,jj)
      &                +coconloc(2)*shpj(2,ii)*shpj(2,jj)
      &                +coconloc(3)*shpj(3,ii)*shpj(3,jj))*weight
@@ -459,7 +472,7 @@ c     &                 weight
 !
 !                     mass matrix
 !
-               if(mass) then
+               if(mass.eq.1) then
                   sm(ii,jj)=sm(ii,jj)
      &                 +c2*shpj(4,ii)*shp(4,jj)
                endif
@@ -469,7 +482,7 @@ c     &                 weight
 !
 !           computation of the right hand side
 !
-         if(rhsi) then
+         if(rhsi.eq.1) then
 !
 !           distributed heat flux
 !
@@ -505,7 +518,8 @@ c     &                 weight
                      call dflux(xload(1,id),t1l,istep,iinc,timeend,
      &                 nelem,kk,pgauss,jltyp,temp,press,sideload(id),
      &                 areaj,vold,co,lakonl,konl,ipompc,nodempc,coefmpc,
-     &                 nmpc,ikmpc,ilmpc,iscale,mi)
+     &                 nmpc,ikmpc,ilmpc,iscale,mi,sti,xstateini,xstate,
+     &                 nstate_,dtime)
                      if((nmethod.eq.1).and.(iscale.ne.0))
      &                    xload(1,id)=xloadold(1,id)+
      &                   (xload(1,id)-xloadold(1,id))*reltime
@@ -520,13 +534,11 @@ c     &                 weight
 !
       enddo
 !
-      if((.not.buckling).and.(nload.ne.0)) then
+      if((buckling.eq.0).and.(nload.ne.0)) then
+         iflag=2
 !
 !       distributed loads
 !
-c         if(nload.eq.0) then
-c            return
-c         endif
          call nident2(nelemload,nelem,nload,id)
          do
             if((id.eq.0).or.(nelemload(1,id).ne.nelem)) exit
@@ -536,7 +548,9 @@ c         endif
                id=id-1
                cycle
             endif
-            read(sideload(id)(2:2),'(i1)') ig
+c            read(sideload(id)(2:2),'(i1)') ig
+            ig=ichar(sideload(id)(2:2))-48
+!
 !
 !         treatment of wedge faces
 !
@@ -626,8 +640,8 @@ c         endif
                 et=gauss2d1(2,i)
                 weight=weight2d1(i)
              elseif((lakonl(4:4).eq.'8').or.
-     &               (lakonl(4:6).eq.'20R').or.
-     &               ((lakonl(4:5).eq.'15').and.(nopes.eq.8))) then
+     &              (lakonl(4:6).eq.'20R').or.
+     &              ((lakonl(4:5).eq.'15').and.(nopes.eq.8))) then
                 xi=gauss2d2(1,i)
                 et=gauss2d2(2,i)
                 weight=weight2d2(i)
@@ -677,30 +691,27 @@ c         endif
                       coords(k)=coords(k)+xl2(k,j)*shp2(4,j)
                    enddo
                 enddo
-                read(sideload(id)(2:2),'(i1)') jltyp
+                jltyp=ichar(sideload(id)(2:2))-48
                 jltyp=jltyp+10
                 if(sideload(id)(1:1).eq.'S') then
                    iscale=1
                    call dflux(xload(1,id),temp,istep,iinc,timeend,
      &               nelem,i,coords,jltyp,temp,press,sideload(id),
      &               areaj,vold,co,lakonl,konl,
-     &               ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,iscale,mi)
-                   if((nmethod.eq.1).and.(iscale.ne.0))
-     &                   xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
+     &               ipompc,nodempc,coefmpc,nmpc,ikmpc,ilmpc,iscale,mi,
+     &               sti,xstateini,xstate,nstate_,dtime)
                 elseif(sideload(id)(1:1).eq.'F') then
                    node=nelemload(2,id)
                    call film(xload(1,id),sinktemp,temp,istep,
      &               iinc,timeend,nelem,i,coords,jltyp,field,nfield,
-     &               sideload(id),node,areaj,vold,mi)
-                   if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
+     &               sideload(id),node,areaj,vold,mi,
+     &               ipkon,kon,lakon,iponoeln,inoeln,ielprop,prop,
+     &               ielmat,shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,
+     &               ncocon,ipobody,xbody,ibody,heatnod,heatfac)
                 elseif(sideload(id)(1:1).eq.'R') then
                    call radiate(xload(1,id),xload(2,id),temp,istep,
      &               iinc,timeend,nelem,i,coords,jltyp,field,nfield,
      &               sideload(id),node,areaj,vold,mi,iemchange)
-                   if(nmethod.eq.1) xload(1,id)=xloadold(1,id)+
-     &                  (xload(1,id)-xloadold(1,id))*reltime
                 endif
              endif
                 
@@ -721,9 +732,8 @@ c         endif
                    ff(ipointer)=ff(ipointer)+shp2(4,k)*xload(1,id)
      &                  *areaj
                 elseif(sideload(id)(1:1).eq.'F') then
-c                   write(*,*) nelem,temp,xload(1,id),sinktemp
-                   ff(ipointer)=ff(ipointer)-shp2(4,k)*xload(1,id)
-     &                  *(temp-sinktemp)*areaj
+                   ff(ipointer)=ff(ipointer)+shp2(4,k)*areaj*
+     &                  (heatfac-xload(1,id)*(temp-sinktemp))
                 elseif(sideload(id)(1:1).eq.'R') then
                    ff(ipointer)=ff(ipointer)-shp2(4,k)*physcon(2)*
      &                  xload(1,id)*((temp-physcon(1))**4-
@@ -750,7 +760,6 @@ c                   write(*,*) nelem,temp,xload(1,id),sinktemp
                    endif
                    if(ipointeri.gt.ipointerj) cycle
                    if(sideload(id)(1:1).eq.'F') then
-c                   write(*,*) nelem,id,xload(1,id),sinktemp
                       s(ipointeri,ipointerj)=s(ipointeri,ipointerj)+
      &                  shp2(4,ii)*shp2(4,jj)*xload(1,id)*areaj
                    elseif(sideload(id)(1:1).eq.'R') then
@@ -770,43 +779,10 @@ c                   write(*,*) nelem,id,xload(1,id),sinktemp
 !     for axially symmetric and plane stress/strain elements: 
 !     complete s and sm
 !
-      if((lakonl(6:7).eq.'RA').or.(lakonl(6:7).eq.'RE')) then
-c      if((lakonl(6:7).eq.'RA').or.(lakonl(6:7).eq.'RS').or.
-c     &   (lakonl(6:7).eq.'RE')) then
-c      if(lakonl(6:7).eq.'RA') then
-         do i=1,20
-            do j=i,20
-               k=iperm(i)
-               l=iperm(j)
-               if(k.gt.l) then
-                  m=k
-                  k=l
-                  l=m
-               endif
-               sax(i,j)=s(k,l)
-            enddo
-         enddo
-         do i=1,20
-            do j=i,20
-               s(i,j)=s(i,j)+sax(i,j)
-            enddo
-         enddo
-!
-!        special treatment of plane stress elements since lateral
-!        heating is allowed (orthogonal to the plane)
-!
-c         if((nload.ne.0).and.(lakonl(6:7).ne.'RS')) then
-         if(nload.ne.0) then
-            do i=1,20
-               k=iperm(i)
-               ffax(i)=ff(k)
-            enddo
-            do i=1,20
-               ff(i)=ff(i)+ffax(i)
-            enddo
-         endif
-!
-         if(mass) then
+c      if(intscheme.eq.0) then
+         if(((lakonl(4:5).eq.'8 ').or.
+     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E'))) then
             do i=1,20
                do j=i,20
                   k=iperm(i)
@@ -816,18 +792,72 @@ c         if((nload.ne.0).and.(lakonl(6:7).ne.'RS')) then
                      k=l
                      l=m
                   endif
-                  sax(i,j)=sm(k,l)
+                  sax(i,j)=s(k,l)
                enddo
             enddo
             do i=1,20
                do j=i,20
-                  sm(i,j)=sm(i,j)+sax(i,j)
+                  s(i,j)=s(i,j)+sax(i,j)
                enddo
             enddo
+!     
+!     special treatment of plane stress elements since lateral
+!     heating is allowed (orthogonal to the plane)
+!     
+            if(nload.ne.0) then
+               do i=1,20
+                  k=iperm(i)
+                  ffax(i)=ff(k)
+               enddo
+               do i=1,20
+                  ff(i)=ff(i)+ffax(i)
+               enddo
+            endif
+!     
+            if(mass.eq.1) then
+               do i=1,20
+                  do j=i,20
+                     k=iperm(i)
+                     l=iperm(j)
+                     if(k.gt.l) then
+                        m=k
+                        k=l
+                        l=m
+                     endif
+                     sax(i,j)=sm(k,l)
+                  enddo
+               enddo
+               do i=1,20
+                  do j=i,20
+                     sm(i,j)=sm(i,j)+sax(i,j)
+                  enddo
+               enddo
+            endif
          endif
-      endif
+c      else
+c!
+c!        only 2-D scheme is reduced for intscheme=1
+c!
+c         if(((lakonl(4:5).eq.'8 ').or.
+c     &        ((lakonl(4:6).eq.'20R').and.(lakonl(7:8).ne.'BR'))).and.
+c     &        ((lakonl(7:7).eq.'A').or.(lakonl(7:7).eq.'E'))) then
+c!     
+c!           special treatment of plane stress elements since lateral
+c!           heating is allowed (orthogonal to the plane)
+c!     
+c            if(nload.ne.0) then
+c               do i=1,20
+c                  k=iperm(i)
+c                  ffax(i)=ff(k)
+c               enddo
+c               do i=1,20
+c                  ff(i)=ff(i)+ffax(i)
+c               enddo
+c            endif
+c         endif
+c      endif
 !
-      if(mass.and.(iexpl.gt.1)) then
+      if((mass.eq.1).and.(iexpl.gt.1)) then
 !
 !        scaling the diagonal terms of the mass matrix such that the total
 !        mass is right (LUMPING; for explicit dynamic calculations)
@@ -849,7 +879,8 @@ c         if((nload.ne.0).and.(lakonl(6:7).ne.'RS')) then
             alp=0.2141d0
          endif
 !
-         if((nope.eq.20).or.(nope.eq.10).or.(nope.eq.15)) then
+         if((nope.eq.20).or.(nope.eq.10).or.
+     &      (nope.eq.15)) then
             factore=summass*alp/(1.d0+alp)/sume
             factorm=summass/(1.d0+alp)/summ
          else
