@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,17 +16,19 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine inputinfo(inpc,ipoinpc,iline)
+      subroutine inputinfo(inpc,ipoinpc,iline,text)
 !
 !     input error message subroutine
 !
       implicit none
 !
       character*1 inpc(*)
+      character*(*) text
 !
       integer ipoinpc(0:*),iline,i
 !
-      write(*,*) '*INFO in the input deck. Card image:'
+      write(*,*) '*INFO reading ',text(1:index(text,'%')-1-1),
+     &      '. Card image:'
       write(*,'(7x,1320a1)') 
      &     (inpc(i),i=ipoinpc(iline-1)+1,ipoinpc(iline))
       write(*,*)
