@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2011 Guido Dhondt
+!     Copyright (C) 1998-2025 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,11 +16,13 @@
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !     
       subroutine labyrinth(node1,node2,nodem,nelem,lakon,
-     &     nactdog,identity,ielprop,prop,iflag,v,xflow,f,
+     &     nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &     nodef,idirf,df,cp,R,physcon,co,dvi,numf,vold,set,
-     &     kon,ipkon,mi)
+     &     kon,ipkon,mi,ttime,time,iaxial,iplausi)
 !     
 !     labyrinth element
+!
+!     author: Yannick Muller
 !     
       implicit none
 !     
@@ -29,23 +31,28 @@
       character*81 set(*)
 !     
       integer nelem,nactdog(0:3,*),node1,node2,nodem,numf,
-     &     ielprop(*),nodef(4),idirf(4),index,iflag,mi(*),
-     &     inv,kgas,n,iaxial,nodea,nodeb,ipkon(*),kon(*),i,itype
+     &     ielprop(*),nodef(*),idirf(*),index,kflag,mi(*),
+     &     inv,kgas,n,iaxial,nodea,nodeb,ipkon(*),kon(*),i,
+     &     itype,iplausi
 !
-      real*8 prop(*),v(0:mi(2),*),xflow,f,df(4),kappa,R,a,d,
-     &     p1,p2,T1,Aeff,C1,C2,C3,cd,cp,physcon(3),p2p1,km1,dvi,
+      real*8 prop(*),v(0:mi(2),*),xflow,f,df(*),kappa,R,a,d,
+     &     p1,p2,T1,Aeff,C1,C2,C3,cd,cp,physcon(*),p2p1,km1,dvi,
      &     kp1,kdkm1,tdkp1,km1dk,x,y,ca1,cb1,ca2,cb2,dT1,alambda,
-     &     rad,reynolds,pi,ppkrit,co(3,*),
-     &     carry_over,lc,hst,e,szt,num,denom,t,s,b,h,cdu,
+     &     rad,reynolds,pi,ppkrit,co(3,*),ttime,time,
+     &     carry_over,dlc,hst,e,szt,num,denom,t,s,b,h,cdu,
      &     cd_radius,cst,dh,cd_honeycomb,cd_lab,bdh,
      &     pt0zps1,cd_1spike,cdbragg,rzdh,
      &     cd_correction,p1p2,xflow_oil,T2,vold(0:mi(2),*)
-!     
+!    
+!
+!
       itype=1
       pi=4.d0*datan(1.d0)
       e=2.718281828459045d0
+!
+      index=ielprop(nelem)
 !    
-      if (iflag.eq.0) then
+      if(kflag.eq.0) then
          identity=.true.
 !     
          if(nactdog(2,node1).ne.0)then
@@ -56,9 +63,12 @@
             identity=.false.
          endif
 !     
-      elseif (iflag.eq.1)then
+      elseif(kflag.eq.1)then
+         if(v(1,nodem).ne.0.d0) then
+            xflow=v(1,nodem)
+            return
+         endif
 !     
-          index=ielprop(nelem)
           kappa=(cp/(cp-R))
 !
 !     Usual Labyrinth
@@ -66,12 +76,11 @@
           if(lakon(nelem)(2:5).ne.'LABF') then
              t=prop(index+1)
              s=prop(index+2)
-             iaxial=int(prop(index+3))
              d=prop(index+4)
-             n=int(prop(index+5))
+             n=nint(prop(index+5))
              b=prop(index+6)
              h=prop(index+7)
-             lc=prop(index+8)
+             dlc=prop(index+8)
              rad=prop(index+9)
              X=prop(index+10)
              Hst=prop(index+11)
@@ -81,16 +90,14 @@
 !    "flexible" labyrinth for thermomechanical coupling
 !
           elseif(lakon(nelem)(2:5).eq.'LABF') then
-             nodea=int(prop(index+1))
-             nodeb=int(prop(index+2))
-             iaxial=int(prop(index+3))
+             nodea=nint(prop(index+1))
+             nodeb=nint(prop(index+2))
              t=prop(index+4)
              d=prop(index+5)
-             n=int(prop(index+6))
+             n=nint(prop(index+6))
              b=prop(index+7)
              h=prop(index+8)
-!     hc=prop(index+7)
-             lc=prop(index+9)
+             dlc=prop(index+9)
              rad=prop(index+10)
              X=prop(index+11)
              Hst=prop(index+12)
@@ -99,23 +106,19 @@
 !     gap definition
              s=dsqrt((co(1,nodeb)+vold(1,nodeb)-
      &            co(1,nodea)-vold(1,nodea))**2)
-             if(iaxial.ne.0) then
-                a=pi*d*s/iaxial
-             else
-                a=pi*d*s
-             endif
+             a=pi*d*s
           endif
 !     
          p1=v(2,node1)
          p2=v(2,node2)
          if(p1.ge.p2) then
             inv=1
-            T1=v(0,node1)+physcon(1)
+            T1=v(0,node1)-physcon(1)
          else
             inv=-1
             p1=v(2,node2)
             p2=v(2,node1)
-            T1=v(0,node2)+physcon(1)
+            T1=v(0,node2)-physcon(1)
          endif
 !     
          cd=1.d0
@@ -125,7 +128,7 @@
 !************************
 !     one fin 
 !*************************
-         if(n.eq.1.d0) then
+         if(n.eq.1) then
 !     
             km1=kappa-1.d0
             kp1=kappa+1.d0
@@ -154,13 +157,13 @@
 !     ASME 98-GT-206
 !**********************
 !     
-         if (n.ge.2) then
+         if(n.ge.2) then
 !     
             call lab_straight_ppkrit(n,ppkrit)
 !     
 !     subcritical case
 !     
-            if (p2p1.gt.ppkrit) then
+            if(p2p1.gt.ppkrit) then
                xflow=inv*p1*Aeff/dsqrt(T1)*dsqrt((1.d0-p2p1**2.d0)
      &              /(R*(n-log(p2p1)/log(e))))
 !     
@@ -171,7 +174,7 @@
             endif
          endif
 !
-      elseif (iflag.eq.2)then
+      elseif(kflag.eq.2)then
          numf=4
          alambda=10000.d0
 !     
@@ -179,9 +182,9 @@
          p2=v(2,node2)
          if(p1.ge.p2) then
             inv=1
-            xflow=v(1,nodem)
-            T1=v(0,node1)+physcon(1)
-            T2=v(0,node2)+physcon(1)
+            xflow=v(1,nodem)*iaxial
+            T1=v(0,node1)-physcon(1)
+            T2=v(0,node2)-physcon(1)
             nodef(1)=node1
             nodef(2)=node1
             nodef(3)=nodem
@@ -190,9 +193,9 @@
             inv=-1
             p1=v(2,node2)
             p2=v(2,node1)
-            xflow=-v(1,nodem)
-            T1=v(0,node2)+physcon(1)
-            T2=v(0,node1)+physcon(1)
+            xflow=-v(1,nodem)*iaxial
+            T1=v(0,node2)-physcon(1)
+            T2=v(0,node1)-physcon(1)
             nodef(1)=node2
             nodef(2)=node2
             nodef(3)=nodem
@@ -207,16 +210,14 @@
 !     Usual labyrinth
 !
          if(lakon(nelem)(2:5).ne. 'LABF') then
-            index=ielprop(nelem)
             kappa=(cp/(cp-R))
             t=prop(index+1)
             s=prop(index+2)
-            iaxial=int(prop(index+3))
             d=prop(index+4)
-            n=int(prop(index+5))
+            n=nint(prop(index+5))
             b=prop(index+6)
             h=prop(index+7)
-            lc=prop(index+8)
+            dlc=prop(index+8)
             rad=prop(index+9)
             X=prop(index+10)
             Hst=prop(index+11)
@@ -225,16 +226,15 @@
 !     Flexible labyrinth for coupled calculations
 !
          elseif(lakon(nelem)(2:5).eq.'LABF') then
-            index=ielprop(nelem)
-            nodea=int(prop(index+1))
-            nodeb=int(prop(index+2))
-            iaxial=int(prop(index+3))
+            nodea=nint(prop(index+1))
+            nodeb=nint(prop(index+2))
+c            iaxial=nint(prop(index+3))
             t=prop(index+4)
             d=prop(index+5)
-            n=int(prop(index+6))
+            n=nint(prop(index+6))
             b=prop(index+7)
             h=prop(index+8)
-            lc=prop(index+9)
+            dlc=prop(index+9)
             rad=prop(index+10)
             X=prop(index+11)
             Hst=prop(index+12)
@@ -242,11 +242,7 @@
 !     gap definition
              s=dsqrt((co(1,nodeb)+vold(1,nodeb)-
      &            co(1,nodea)-vold(1,nodea))**2)
-             if(iaxial.ne.0) then
-                a=pi*d*s/iaxial
-             else
-                a=pi*d*s
-             endif
+             a=pi*d*s
           endif
 !     
          p2p1=p2/p1
@@ -257,8 +253,8 @@
 !     honeycomb stator correction
 !     
          cd_honeycomb=1.d0
-         if (lc.ne.0.d0)then
-            call cd_lab_honeycomb(s,lc,cd_honeycomb)
+         if(dlc.ne.0.d0)then
+            call cd_lab_honeycomb(s,dlc,cd_honeycomb)
             cd_honeycomb=1+cd_honeycomb/100
          endif
 !     
@@ -269,20 +265,22 @@
             call cd_lab_radius(rad,s,Hst,cd_radius)
          endif
 !     
-!     carry over factor (only for straight throught labyrinth)
+!     carry over factor (only for straight through labyrinth)
 !     
-         if ((n.ge.2).and.(hst.eq.0d0)) then
+         if((n.ge.2).and.(hst.eq.0.d0)) then
             cst=n/(n-1.d0)
             szt=s/t
-            carry_over=cst/dsqrt(cst-szt/(szt+0.02))
+            carry_over=cst/dsqrt(cst-szt/(szt+0.02d0))
             Aeff=Aeff*carry_over
          endif
 !     
 !     calculation of the dynamic viscosity 
 !     
-         if(dabs(dvi).lt.1E-30) then
-            kgas=0
-            call dynamic_viscosity(kgas,T1,dvi)
+         if(dabs(dvi).lt.1d-30) then
+            write(*,*) '*ERROR in labyrinth: '
+            write(*,*) '       no dynamic viscosity defined'
+            write(*,*) '       dvi= ',dvi
+            call exit(201)
          endif     
 !     
 !     calculation of the number of reynolds for a gap
@@ -385,11 +383,11 @@
 !     corrective term for the first spike
                p1p2=p1/p2
                pt0zps1=(p1p2)**(1/prop(index+4))
-               call cd_lab_1spike (pt0zps1,s,b,cd_1spike)
+               call cd_lab_1spike(pt0zps1,s,b,cd_1spike)
 !     
 !     corrective term for cd_lab_1spike
 !     
-               call cd_lab_correction (p1p2,s,b,cd_correction)
+               call cd_lab_correction(p1p2,s,b,cd_correction)
 !     
 !     calculation of the discharge coefficient of the stepped labyrinth
 !     
@@ -403,7 +401,7 @@
 !     
 !     subcritical case
 !     
-            if (p2p1.gt.ppkrit) then
+            if(p2p1.gt.ppkrit) then
 !     
                f=xflow*dT1/p1-dsqrt(num/denom)*Aeff
 !     
@@ -430,16 +428,15 @@
 !
 !     output
 !
-      elseif(iflag.eq.3)then
+      elseif(kflag.eq.3)then
 !
-
          p1=v(2,node1)
          p2=v(2,node2)
          if(p1.ge.p2) then
             inv=1
-            xflow=v(1,nodem)
-            T1=v(0,node1)+physcon(1)
-            T2=v(0,node2)+physcon(1)
+            xflow=v(1,nodem)*iaxial
+            T1=v(0,node1)-physcon(1)
+            T2=v(0,node2)-physcon(1)
             nodef(1)=node1
             nodef(2)=node1
             nodef(3)=nodem
@@ -448,24 +445,23 @@
             inv=-1
             p1=v(2,node2)
             p2=v(2,node1)
-            xflow=-v(1,nodem)
-            T1=v(0,node2)+physcon(1)
-            T2=v(0,node2)+physcon(1)
+            xflow=-v(1,nodem)*iaxial
+            T1=v(0,node2)-physcon(1)
+            T2=v(0,node2)-physcon(1)
             nodef(1)=node2
             nodef(2)=node2
             nodef(3)=nodem
             nodef(4)=node1
          endif
 !     
-         index=ielprop(nelem)
          kappa=(cp/(cp-R))
          t=prop(index+1)
          s=prop(index+2)
          d=prop(index+3)
-         n=int(prop(index+4))
+         n=nint(prop(index+4))
          b=prop(index+5)
          h=prop(index+6)
-         lc=prop(index+7)
+         dlc=prop(index+7)
          rad=prop(index+8)
          X=prop(index+9)
          Hst=prop(index+10)
@@ -480,8 +476,8 @@
 !     
 !     honeycomb stator correction
 !     
-         if (lc.ne.0.d0)then
-            call cd_lab_honeycomb(s,lc,cd_honeycomb)
+         if(dlc.ne.0.d0)then
+            call cd_lab_honeycomb(s,dlc,cd_honeycomb)
             Aeff=Aeff*(1.d0+cd_honeycomb/100.d0)
          else
             cd_honeycomb=0
@@ -496,20 +492,22 @@
             cd_radius=1
          endif
 !     
-!     carry over factor (only for straight throught labyrinth)
+!     carry over factor (only for straight through labyrinth)
 !     
-         if((n.gt.1).and.(hst.eq.0d0)) then
+         if((n.gt.1).and.(hst.eq.0.d0)) then
             cst=n/(n-1.d0)
             szt=s/t
-            carry_over=cst/dsqrt(cst-szt/(szt+0.02))
+            carry_over=cst/dsqrt(cst-szt/(szt+0.02d0))
             Aeff=Aeff*carry_over
          endif
 !     
 !     calculation of the dynamic viscosity 
 !     
-         if(dabs(dvi).lt.1E-30) then
-            kgas=0
-            call dynamic_viscosity(kgas,T1,dvi)
+         if(dabs(dvi).lt.1d-30) then
+            write(*,*) '*ERROR in labyrinth: '
+            write(*,*) '       no dynamic viscosity defined'
+            write(*,*) '       dvi= ',dvi
+            call exit(201)
          endif     
 !     
 !     calculation of the number of reynolds for a gap
@@ -570,11 +568,11 @@
 !     corrective term for the first spike
                p1p2=p1/p2
                pt0zps1=(p1p2)**(1/prop(index+4))
-               call cd_lab_1spike (pt0zps1,s,b,cd_1spike)
+               call cd_lab_1spike(pt0zps1,s,b,cd_1spike)
 !     
 !     corrective term for cd_lab_1spike
 !     
-               call cd_lab_correction (p1p2,s,b,cd_correction)
+               call cd_lab_correction(p1p2,s,b,cd_correction)
 !     
 !     calculation of the discharge coefficient of the stepped labyrinth
 !     
@@ -585,26 +583,26 @@
             endif
 !     
             call lab_straight_ppkrit(n,ppkrit)
-
+!
          endif
-
+!
          xflow_oil=0
-
+!
          write(1,*) ''
-         write(1,55) 'In line',int(nodem/100),' from node',node1,
-     &' to node', node2,':   air massflow rate= ',xflow,'kg/s',
-     &', oil massflow rate= ',xflow_oil,'kg/s'
- 55      FORMAT(1X,A,I6.3,A,I6.3,A,I6.3,A,F9.6,A,A,F9.6,A)
+         write(1,55) ' from node',node1,
+     &' to node', node2,':   air massflow rate= ',xflow,
+     &', oil massflow rate= ',xflow_oil
+ 55      FORMAT(1X,A,I6,A,I6,A,e11.4,A,A,e11.4,A)
          
          if(inv.eq.1) then
-          write(1,56)'       Inlet node  ',node1,':   Tt1=',T1,
-     &           'K, Ts1=',T1,'K, Pt1=',P1/1E5, 'Bar'
+          write(1,56)'       Inlet node ',node1,':   Tt1=',T1,
+     &           ', Ts1=',T1,', Pt1=',p1
 
-            write(1,*)'             element S    ',set(numf)(1:20)
-            write(1,57)'             eta= ',dvi,'kg/(m*s), Re= ' ,
+            write(1,*)'             Element ',nelem,lakon(nelem)
+            write(1,57)'             dyn.visc.= ',dvi,', Re= ' ,
      &           reynolds,
      &', Cd_radius= ',cd_radius,', Cd_honeycomb= ', 1+cd_honeycomb/100
-
+!
 !     straight labyrinth
            if((hst.eq.0.d0).and.(n.ne.1)) then
               write(1,58)'             COF= ',carry_over,
@@ -622,18 +620,18 @@
      &             ', Cd= ',cdbragg
            endif
                  
-            write(1,56)'       Outlet node ',node2,':   Tt2= ',T2,
-     &           'K, Ts2= ',T2,'K, Pt2= ',P2/1e5,'Bar'
+            write(1,56)'      Outlet node ',node2,':   Tt2= ',T2,
+     &           ', Ts2= ',T2,', Pt2= ',p2
 
 !     
          else if(inv.eq.-1) then
-            write(1,56)'       Inlet node  ',node2,':    Tt1= ',T1,
-     &           'K, Ts1= ',T1,'K, Pt1= ',P1/1E5, 'Bar' 
+            write(1,56)'       Inlet node ',node2,':    Tt1= ',T1,
+     &           ', Ts1= ',T1,', Pt1= ',p1 
          
-            write(1,*)'             element S    ',set(numf)(1:20)
-            write(1,57)'             eta=',dvi,'kg/(m*s), Re= '
+            write(1,*)'             element ',nelem,lakon(nelem)
+            write(1,57)'             dyn.visc.=',dvi,', Re= '
      &           ,reynolds,
-     & ', Cd_radius= ',cd_radius,', Cd_honeycomb= ',1+cd_honeycomb/100       
+     & ', Cd_radius= ',cd_radius,', Cd_honeycomb= ',1+cd_honeycomb/100
 !
 !     straight labyrinth
             if((hst.eq.0.d0).and.(n.ne.1)) then
@@ -651,19 +649,20 @@
                write(1,60) '              Cd_Mcgreehan= ',
      & cdu,' Cd= ',cdbragg
            endif
-           write(1,56)'       Outlet node ',node1,':    Tt2= ',T2,
-     &          'K, Ts2= ',T2,'K, Pt2= ',P2/1e5, 'Bar'
+           write(1,56)'      Outlet node ',node1,':    Tt2= ',T2,
+     &          ', Ts2= ',T2,', Pt2= ',p2
 
         endif
 !         
- 56      FORMAT(1X,A,I6.3,A,f6.1,A,f6.1,A,f9.5,A)
- 57      FORMAT(1X,A,E11.5,A,G9.4,A,f6.4,A,f6.4)
- 58      FORMAT(1X,A,f7.5,A,f7.5,A,f7.5)
- 59      FORMAT(1X,A,f7.5,A,f7.5,A,f7.5,A,f5.3)
- 60      FORMAT(1X,A,f7.5,A,f7.5)
+ 56      FORMAT(1X,A,I6,A,e11.4,A,e11.4,A,e11.4,A)
+ 57      FORMAT(1X,A,E11.5,A,e11.4,A,e11.4,A,e11.4)
+ 58      FORMAT(1X,A,e11.4,A,e11.4,A,e11.4)
+ 59      FORMAT(1X,A,e11.4,A,e11.4,A,e11.4,A,e11.4)
+ 60      FORMAT(1X,A,e11.4,A,e11.4)
       endif
+!     
+      xflow=xflow/iaxial
+      df(3)=df(3)*iaxial
 !         
       return
       end
-      
-      
