@@ -1,6 +1,6 @@
 !     
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2011 Guido Dhondt
+!     Copyright (C) 1998-2025 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -23,42 +23,49 @@
      &     istep,dtime,ttime,time,
      &     ielmat,nteq,prop,ielprop,nactdog,nacteq,physcon,
      &     rhcon,nrhcon,ipobody,ibody,xbodyact,nbody,vold,xloadold,
-     &     reltime,nmethod,set,mi,nmpc,nodempc,ipompc,coefmpc,labmpc)
+     &     reltime,nmethod,set,mi,nmpc,nodempc,ipompc,coefmpc,labmpc,
+     &     iaxial,cocon,ncocon,iponoeln,inoeln)
 !     
       implicit none
 !     
-      logical identity
+      logical identity,crit
+!
       character*8 lakonl,lakon(*)
       character*20 sideload(*),labmpc(*)
       character*81 set(*)
 !     
-      integer mi(*),itg(*),ieg(*),ntg,nteq,nflow,nload,ielmat(mi(3),*),
+      integer mi(*),itg(*),ieg(*),ntg,nteq,nflow,nload,inv,
+     &     ielmat(mi(3),*),iaxial,ncocon(2,*),iponoeln(*),inoeln(2,*),
      &     nelemload(2,*),nope,nopes,mint2d,i,j,k,l,iflag,
      &     node,imat,ntmat_,id,ifaceq(8,6),ifacet(6,4),
      &     ifacew(8,5),node1,node2,nshcon(*),nelem,ig,index,konl(20),
      &     ipkon(*),kon(*),idof,iinc,ibody(3,*),istep,jltyp,nfield,
-     &     ipobody(2,*),nodem,ieq,kflag,nrhcon(*),numf,
+     &     ipobody(2,*),nodem,ieq,kflag,nrhcon(*),numf,k_oil,
      &     idofp1,idofp2,idofm,idoft1,idoft2,idoft,nactdog(0:3,*),
      &     nacteq(0:3,*),ielprop(*),nodef(8),idirf(8),nbody,
-     &     nmethod,icase,nmpc,nodempc(3,*),ipompc(*),idir,ider
+     &     nmethod,icase,nmpc,nodempc(3,*),ipompc(*),idir,ider,
+     &     iplausi
 !     
       real*8 ac(nteq,*),xloadact(2,*),cp,h(2),physcon(*),dvi,
      &     xl2(3,8),coords(3),dxsj2,temp,xi,et,weight,xsj2(3),
      &     gastemp,v(0:mi(2),*),shcon(0:3,ntmat_,*),co(3,*),shp2(7,8),
-     &     ftot,field,prop(*),f,df(8),tg1,tg2,r,rho,tl2(8),
+     &     ftot,field,prop(*),f,df(8),tg1,tg2,r,rho,tl2(8),dl,
      &     dtime,ttime,time,areaj,xflow,tvar(2),g(3),coefmpc(*),
      &     rhcon(0:1,ntmat_,*),xbodyact(7,*),sinktemp,ts1,ts2,xs2(3,7),
-     &     xdenom1,xdenom2,xcst,xk1,xk2,expon,a,dt1,dt2,kappa,
-     &     pt1,pt2,inv,vold(0:mi(2),*),xloadold(2,*),reltime,pi
+     &     pt1,pt2,vold(0:mi(2),*),xloadold(2,*),reltime,pi,d,
+     &     cocon(0:6,ntmat_,*),xflow360,xflow_oil,ks,form_fact,
+     &     Qred_crit,reynolds,pt2zpt1_c,Qred1,pt2zpt1,phi,lambda,
+     &     M1,M2,bb,cc,km1,kp1,A,kappa,ff1,ff2,Tt1,Tt2,T1,T2,M1_c,
+     &     heatnod,heatfac
 !     
       include "gauss.f"
 !     
       data ifaceq /4,3,2,1,11,10,9,12,
-     &     5,6,7,8,13,14,15,16,
-     &     1,2,6,5,9,18,13,17,
-     &     2,3,7,6,10,19,14,18,
-     &     3,4,8,7,11,20,15,19,
-     &     4,1,5,8,12,17,16,20/
+     &            5,6,7,8,13,14,15,16,
+     &            1,2,6,5,9,18,13,17,
+     &            2,3,7,6,10,19,14,18,
+     &            3,4,8,7,11,20,15,19,
+     &            4,1,5,8,12,17,16,20/
       data ifacet /1,3,2,7,6,5,
      &     1,2,4,5,9,8,
      &     2,3,4,6,10,9,
@@ -76,7 +83,7 @@
 !
       Pi=4.d0*datan(1.d0)
       tvar(1)=time
-      tvar(2)=ttime+dtime
+      tvar(2)=ttime+time
 !     
 !     reinitialisation of the Ac matrix
 !
@@ -92,7 +99,6 @@
 !     
 !     element contribution.
 !              
- 
       do i=1,nflow
          nelem=ieg(i)
          index=ipkon(nelem)
@@ -101,6 +107,8 @@
          node2=kon(index+3)
 !
          xflow=v(1,nodem)
+!
+!        total/static temperatures
 !
         if((lakon(nelem)(2:3).ne.'LP').and.
      &     (lakon(nelem)(2:3).ne.'LI')) then
@@ -124,7 +132,7 @@
             gastemp=(ts1+ts2)/2.d0
 !
 !     for liquid pipe element only the upstream temperature is used to 
-!     determine thematerial properties
+!     determine the material properties
 !
          else
             
@@ -141,6 +149,17 @@
                   gastemp=v(0,node2)
                endif
             endif
+!
+            if(node1.eq.0) then
+               tg2=v(0,node2)
+               tg1=tg2
+            elseif(node2.eq.0) then
+               tg1=v(0,node1)
+               tg2=tg1
+            else
+               tg1=v(0,node1)
+               tg2=v(0,node2)
+            endif
          endif
 !
          imat=ielmat(1,nelem)
@@ -149,61 +168,6 @@
      &        rhcon,nrhcon,rho)
 !     
          kappa=(cp/(cp-R))
-!
-!     Definitions of the constant for isothermal flow elements
-!
-         if((lakon(nelem)(2:6).eq.'GAPFI')
-     &        .or.(lakon(nelem)(2:6).eq.'GAPII'))then
-            if((node1.ne.0).and.(node2.ne.0)) then
-!
-               icase=1
-               A=prop(ielprop(nelem)+1)
-               pt1=v(2,node1)
-               pt2=v(2,node2)
-               if(pt1.ge.pt2)then
-                  inv=1.d0
-                  pt1=v(2,node1)
-                  pt2=v(2,node2)
-                  if(dabs(tg2/ts2-(1+0.5*(kappa-1)/kappa)).lt.1E-5) then
-                    
-                     pt2=dabs(xflow)*dsqrt(Tg2*R)/A
-     &                    *(1+0.5*(kappa-1)/kappa)
-     &                    **(0.5*(kappa+1)/(kappa-1)) 
-                  endif
-                  tg1=v(0,node1)
-                  call ts_calc(xflow,Tg1,Pt1,kappa,r,a,Ts1,icase)
-                  
-
-                  tg2=v(0,node2)
-                  call ts_calc(xflow,Tg2,Pt2,kappa,r,a,Ts2,icase)
-               else
-                 
-                  inv=-1.d0
-                  pt1=v(2,node2)
-                  pt2=v(2,node1)
-                  if(dabs(tg2/ts2-(1+0.5*(kappa-1)/kappa)).lt.1E-5) then
-                    
-                     pt2=dabs(xflow)*dsqrt(Tg2*R)/A
-     &                    *(1+0.5*(kappa-1)/kappa)
-     &                    **(0.5*(kappa+1)/(kappa-1)) 
-                  endif
-                  tg1=v(0,node2)
-                  call ts_calc(xflow,Tg1,Pt1,kappa,r,a,Ts1,icase)
-                  tg2=v(0,node1)
-                  call ts_calc(xflow,Tg2,Pt2,kappa,r,a,Ts2,icase)
-               endif
-               dt1=tg1/ts1-1d0
-               dt2=tg2/ts2-1d0
-               expon=2.d0*kappa/(kappa-1.d0)
-               xcst=2.d0*Cp*A**2/r**2
-               xk1=pt1**2*(ts1/tg1)**expon
-               xdenom1=xcst*xk1*(1.d0-expon*(tg1/ts1-1.d0))
-     &              /ts1+2.d0*xflow**2
-               xk2=pt2**2*(ts2/tg2)**expon
-               xdenom2=xcst*xk2*(1.d0-expon*(tg2/ts2-1.d0))
-     &              /ts2+2.d0*xflow**2
-            endif
-         endif
 !     
          if(node1.ne.0) then
             idoft1=nactdog(0,node1)
@@ -220,16 +184,74 @@
             idofp2=0
          endif
          idofm=nactdog(1,nodem)
+!
+!     Definitions of the constant for isothermal flow elements
+!
+         if(lakon(nelem)(2:6).eq.'GAPFI') then
+            if((node1.ne.0).and.(node2.ne.0)) then
+               icase=1
+!
+!              properties
+!
+               index=ielprop(nelem)
+               A=prop(index+1)
+               d=prop(index+2)
+               dl=prop(index+3)
+               ks=prop(index+4)
+               form_fact=prop(index+5)
+               xflow_oil=prop(index+6)
+               k_oil=nint(prop(index+7))
+!
+               km1=kappa-1.d0
+               kp1=kappa+1.d0
+!
+c               xflow360=xflow*iaxial
+!
+!              orientation of the element based on the direction of the
+!              mass flow; determination of the total pressures and total
+!              temperatures (K)
+!
+!              xflow360 is the absolute value of the mass flow for
+!              360 degrees
+!
+c               if(xflow.gt.0.d0) then
+               if(v(2,node1).gt.v(2,node2)) then
+                  inv=1
+                  xflow360=xflow*iaxial
+                  pt1=v(2,node1)
+                  pt2=v(2,node2)
+                  Tt1=v(0,node1)-physcon(1)
+                  Tt2=v(0,node2)-physcon(1)
+               else
+                  inv=-1
+                  xflow360=-xflow*iaxial
+                  pt1=v(2,node2)
+                  pt2=v(2,node1)
+                  Tt1=v(0,node2)-physcon(1)
+                  Tt2=v(0,node1)-physcon(1)
+!     
+                  idoft1=nactdog(0,node2)
+                  idofp1=nactdog(2,node2)
+                  idoft2=nactdog(0,node1)
+                  idofp2=nactdog(2,node1)
+               endif
+!
+!              calculation of the static temperatures T1 and T2
+!              (K)
+!
+               call ts_calc(xflow360,Tt1,pt1,kappa,r,A,T1,icase)
+            endif
+         endif
 !     
          if(node1.ne.0) then
 !     
 !     energy equation contribution node1
 !     
-            if (nacteq(0,node1).ne.0) then
+            if(nacteq(0,node1).ne.0) then
                ieq=nacteq(0,node1)
-               if ((xflow.le.0d0).and.(nacteq(3,node1).eq.0))then
+               if((xflow.le.0.d0).and.(nacteq(3,node1).eq.0))then
 !     
-!     adiabatic element
+!                 incoming flow in node1 and genuine energy conservation equation
 !     
                   if(idoft1.ne.0) then
                      ac(ieq,idoft1)=ac(ieq,idoft1)-cp*xflow
@@ -237,74 +259,139 @@
 !     
                   if(idoft2.ne.0)then
                      ac(ieq,idoft2)=ac(ieq,idoft2)+cp*xflow
-                  endif
-!     
-                  if(idofm.ne.0) then
-                     ac(ieq,idofm)=ac(ieq,idofm)-cp*(tg1-tg2)
+                  elseif(node2.eq.0) then
+                     write(*,*) '*WARNING in mafillnet'
+                     write(*,*) '         temperature at inlet'
+                     write(*,*) '         node',node1,' unknown'
+c                     call exit(201)
                   endif
 !
-               elseif(nacteq(3,node1).ne.0)then
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
 !     
-!     isothermal element
+                  if(idofm.ne.0) then
+c                     ac(ieq,idofm)=ac(ieq,idofm)-cp*(tg1-tg2)
+                     ac(ieq,idofm)=0.d0
+                  endif
+!
+               elseif((node2.ne.0).and.(nacteq(3,node1).eq.node2)) then
 !     
-                  if(nacteq(3,node1).eq.node2) then
+!                 isothermal element
 !     
-                     if(inv.eq.-1d0) then
-                        if(idoft1.ne.0) then
-                           ac(ieq,idoft1)=-xcst*xk1*(1.d0-expon
-     &                          *(1.d0-ts1/tg1))/(xdenom1*ts1) 
-                        endif
+                  pt2zpt1=pt2/pt1
 !     
-                        if(idoft2.ne.0)then
-                           ac(ieq,idoft2)=xcst*xk2*(1.d0-expon
-     &                          *(1.d0-ts2/tg2))/(xdenom2*ts2) 
-                        endif
+!     calculation of the dynamic viscosity
 !     
-                        if(idofm.ne.0) then
-                           ac(ieq,idofm)=(-2.d0*xflow*ts2
-     &                          /xdenom2+2.d0*xflow*ts1/xdenom1)
-                        endif
+                  if(dabs(dvi).lt.1d-30) then
+                     write(*,*) '*ERROR in mafillnet: '
+                     write(*,*) '       no dynamic viscosity defined'
+                     write(*,*) '       dvi= ',dvi
+                     call exit(201)
+                  endif        
 !     
-                        if(idofp1.ne.0) then
-                           ieq=nacteq(2,idofp1)
-                           ac(ieq,idofp1)=2.d0*xcst*dt1*xk1
-     &                          /(pt1*xdenom1)
-                        endif
+                  reynolds=xflow360*d/(dvi*A)
+                  if(reynolds.lt.1) then
+                     reynolds=1.d0
+                  endif
 !     
-                        if(idofp2.ne.0) then
-                           ac(ieq,idofp2)=-2.d0*xcst*dt2*xk2
-     &                          /(pt2*xdenom2)
-                        endif
+!                 calculation of the friction coefficient
 !     
-                     elseif(inv.eq.1d0)then
-                        if(idoft1.ne.0) then
-                           ac(ieq,idoft1)=xcst*xk1*(1.d0-expon
-     &                          *(1.d0-ts1/tg1))/(xdenom1*ts1) 
-                        endif
+                  if(xflow_oil.ne.0d0) then
 !     
-                        if(idoft2.ne.0)then
-                           ac(ieq,idoft2)=-xcst*xk2*(1.d0-expon
-     &                          *(1.d0-ts2/tg2))/(xdenom2*ts2) 
-                        endif
+!                    two-phase-flow
 !     
-                        if(idofm.ne.0) then
-                           ac(ieq,idofm)=-(-2.d0*xflow*ts2
-     &                          /xdenom2+2.d0*xflow*ts1/xdenom1)
-                        endif
+                     call two_phase_flow(Tt1,pt1,T1,Tt2,pt2,T2,xflow360,
+     &                    xflow_oil,nelem,lakon,kon,ipkon,ielprop,prop,
+     &                    v,dvi,cp,r,k_oil,phi,lambda,nshcon,nrhcon,
+     &                    shcon,rhcon,ntmat_,mi)
+                     lambda=lambda*phi
+                  else
 !     
-                        if(idofp1.ne.0) then
-                           ieq=nacteq(2,idofp1)
-                           ac(ieq,idofp1)=-2.d0*xcst*dt1*xk1
-     &                          /(pt1*xdenom1)
-                        endif
+!                    for pure air
 !     
-                        if(idofp2.ne.0) then
-                           ac(ieq,idofp2)=2.d0*xcst*dt2*xk2
-     &                          /(pt2*xdenom2)
-                        endif
+                     phi=1.d0
+                     call friction_coefficient(dl,d,ks,reynolds,
+     &                    form_fact,lambda)
+                  endif
+!
+!                 calculating the critical conditions
 !     
+                  call pt2zpt1_crit(pt2,pt1,Tt1,lambda,kappa,r,dl,d,
+     &                 pt2zpt1_c,Qred_crit,crit,icase,M1_c)
+!
+                  Qred1=xflow360*dsqrt(Tt1)/(A*pt1)
+!
+!                 check whether flow is critical
+!                 assigning the physical correct sign to xflow360
+!
+                  if(crit) then
+!
+!                    critical flow
+!
+                     xflow360=inv*Qred_crit*A*pt1/dsqrt(Tt1)
+                     M1=dsqrt(2/km1*((Tt1/T1)-1.d0))
+c                     M1=min(M1,0.999d0/dsqrt(kappa))
+                  else
+!
+!                    subcritical flow
+!
+                     if(Qred1.gt.Qred_crit) then
+                        xflow360=inv*Qred_crit*A*pt1/dsqrt(Tt1)
+                        M1=M1_c
+                     else
+                        xflow360=inv*xflow360
+                        M1=dsqrt(2/km1*((Tt1/T1)-1.d0))
+                     endif
+                     call ts_calc(xflow360,Tt2,pt2,kappa,r,A,T2,icase)
+                     M2=dsqrt(2/km1*((Tt2/T2)-1.d0))
+                  endif
+!     
+                  bb=km1/2.d0
+                  cc=-kp1/(2.d0*km1)
+                  ff1=(2.d0*bb*M1**2)/(1.d0+bb*M1**2*(1.d0+2.d0*cc))
+!
+                  if(.not.crit) then
+                     ff2=(2.d0*bb*M2**2)/(1.d0+bb*M2**2*(1.d0+2.d0*cc))
+                     if(idofp1.ne.0) then
+                        ac(ieq,idofp1)=ff1*T1/pt1
+                     endif
+                     if(idoft1.ne.0) then
+                        ac(ieq,idoft1)=(1.d0-ff1/2.d0)/(1.d0+bb*M1**2)
+                     endif
+!
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
+!     
+                     if(idofm.ne.0) then
+c                        ac(ieq,idofm)=(ff2*T2-ff1*T1)/xflow360
+                        ac(ieq,idofm)=0.d0
+                     endif
+                     if(idofp2.ne.0) then
+                        ac(ieq,idofp2)=-ff2*T2/pt2
+                     endif
+                     if(idoft2.ne.0) then
+                        ac(ieq,idoft2)=(ff2/2.d0-1.d0)/(1.d0+bb*M2**2)
+                     endif
+                   else
+                     if(idofp1.ne.0) then
+                        ac(ieq,idofp1)=ff1*T1/pt1
+                     endif
+                     if(idoft1.ne.0) then
+                        ac(ieq,idoft1)=(1.d0-ff1/2.d0)/(1.d0+bb*M1**2)
+                     endif
+!
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
+!     
+                     if(idofm.ne.0) then
+c                        ac(ieq,idofm)=-ff1*T1/xflow360
+                        ac(ieq,idofm)=0.d0
+                     endif
+                     if(idoft2.ne.0) then
+                        ac(ieq,idoft2)=-1.d0/(1.d0+bb/kappa)
                      endif
                   endif
+!
                endif
             endif
 !     
@@ -320,84 +407,154 @@
 !     
          if(node2.ne.0) then
 !     
-!     energy equation contribution node2
+!           energy equation contribution node2
 !     
-            if (nacteq(0,node2).ne.0) then
+            if(nacteq(0,node2).ne.0) then
                ieq=nacteq(0,node2)
-               if ((xflow.ge.0d0).and.(nacteq(3,node2).eq.0))then
+               if((xflow.ge.0d0).and.(nacteq(3,node2).eq.0))then
 !
-!     adiabatic element
+!                 adiabatic element
 !     
                   if(idoft1.ne.0)then
                      ac(ieq,idoft1)=ac(ieq,idoft1)-cp*xflow
+                  elseif(node1.eq.0) then
+                     write(*,*) '*WARNING in mafillnet'
+                     write(*,*) '         temperature at inlet'
+                     write(*,*) '         node',node2,' unknown'
+c                     call exit(201)
                   endif
 !     
                   if(idoft2.ne.0) then
                      ac(ieq,idoft2)=ac(ieq,idoft2)+cp*xflow
                   endif
+!     
+!
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
 !     
                   if(idofm.ne.0) then
-                     ac(ieq,idofm)=ac(ieq,idofm)+cp*(tg2-tg1)
+c                     ac(ieq,idofm)=ac(ieq,idofm)+cp*(tg2-tg1)
+                     ac(ieq,idofm)=0.d0
                   endif
 !     
-               elseif((nacteq(3,node2).eq.node1))then
+               elseif((node1.ne.0).and.(nacteq(3,node2).eq.node1))then
 !     
-!     isothermal element
+!                 isothermal element
 !     
-                  if(inv.eq.-1d0) then
-                     if(idoft1.ne.0)then
-                        ac(ieq,idoft1)=-xcst*xk1*(1.d0-expon
-     &                       *(1.d0-ts1/tg1))/(xdenom1*ts1) 
-                     endif
-!     
-                     if(idoft2.ne.0) then
-                        ac(ieq,idoft2)=(xcst*xk2*(1.d0-expon
-     &                       *(1.d0-ts2/tg2))/(xdenom2*ts2)) 
-                     endif
-!     
-                     if(idofm.ne.0) then
-                        ac(ieq,idofm)=(-2.d0*xflow*ts2
-     &                       /xdenom2+2.d0*xflow*ts1/xdenom1)
+                  pt2zpt1=pt2/pt1
+!     
+!     calculation of the dynamic viscosity
+!     
+                  if(dabs(dvi).lt.1d-30) then
+                     write(*,*) '*ERROR in mafillnet: '
+                     write(*,*) '       no dynamic viscosity defined'
+                     write(*,*) '       dvi= ',dvi
+                     call exit(201)
+                  endif        
+!     
+                  reynolds=xflow360*d/(dvi*A)
+                  if(reynolds.lt.1) then
+                     reynolds=1.d0
+                  endif
+!     
+!                 calculation of the friction coefficient
+!     
+                  if(xflow_oil.ne.0d0) then
+!     
+!                    two-phase-flow
+!     
+                     call two_phase_flow(Tt1,pt1,T1,Tt2,pt2,T2,xflow360,
+     &                    xflow_oil,nelem,lakon,kon,ipkon,ielprop,prop,
+     &                    v,dvi,cp,r,k_oil,phi,lambda,nshcon,nrhcon,
+     &                    shcon,rhcon,ntmat_,mi)
+                     lambda=lambda*phi
+                  else
+!     
+!                    for pure air
+!     
+                     phi=1.d0
+                     call friction_coefficient(dl,d,ks,reynolds,
+     &                    form_fact,lambda)
+                  endif
+!
+!                 calculating the critical conditions
+!     
+                  call pt2zpt1_crit(pt2,pt1,Tt1,lambda,kappa,r,dl,d,
+     &                 pt2zpt1_c,Qred_crit,crit,icase,M1_c)
+!
+                  Qred1=xflow360*dsqrt(Tt1)/(A*pt1)
+!
+!                 check whether flow is critical
+!                 assigning the physical correct sign to xflow360
+!
+                  if(crit) then
+!
+!                    critical flow
+!
+                     xflow360=inv*Qred_crit*A*pt1/dsqrt(Tt1)
+                     M1=dsqrt(2/km1*((Tt1/T1)-1.d0))
+c                     M1=min(M1,0.999d0/dsqrt(kappa))
+                  else
+!
+!                    subcritical flow
+!
+                     if(Qred1.gt.Qred_crit) then
+                        xflow360=inv*Qred_crit*A*pt1/dsqrt(Tt1)
+                        M1=M1_c
+                     else
+                        xflow360=inv*xflow360
+                        M1=dsqrt(2/km1*((Tt1/T1)-1.d0))
                      endif
+                     call ts_calc(xflow360,Tt2,pt2,kappa,r,A,T2,icase)
+                     M2=dsqrt(2/km1*((Tt2/T2)-1.d0))
+                  endif
 !     
+                  bb=km1/2.d0
+                  cc=-kp1/(2.d0*km1)
+                  ff1=(2.d0*bb*M1**2)/(1.d0+bb*M1**2*(1.d0+2.d0*cc))
+!
+                  if(.not.crit) then
+                     ff2=(2.d0*bb*M2**2)/(1.d0+bb*M2**2*(1.d0+2.d0*cc))
                      if(idofp1.ne.0) then
-                        ac(ieq,idofp1)=+2.d0*xcst*dt1*xk1
-     &                       /(pt1*xdenom1)
+                        ac(ieq,idofp1)=ff1*T1/pt1
                      endif
-!     
-                     if(idofp2.ne.0) then
-                        ac(ieq,idofp2)=-2.d0*xcst*dt2*xk2
-     &                       /(pt2*xdenom2)
+                     if(idoft1.ne.0) then
+                        ac(ieq,idoft1)=(1.d0-ff1/2.d0)/(1.d0+bb*M1**2)
                      endif
 !
-                  elseif(inv.eq.1d0) then
-     
-                     if(idoft1.ne.0)then
-                        ac(ieq,idoft1)=xcst*xk1*(1.d0-expon
-     &                       *(1.d0-ts1/tg1))/(xdenom1*ts1) 
-                     endif
-!     
-                     if(idoft2.ne.0) then
-                        ac(ieq,idoft2)=-(xcst*xk2*(1.d0-expon
-     &                       *(1.d0-ts2/tg2))/(xdenom2*ts2)) 
-                     endif
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
 !     
                      if(idofm.ne.0) then
-                        ac(ieq,idofm)=-(-2.d0*xflow*ts2
-     &                       /xdenom2+2.d0*xflow*ts1/xdenom1)
+c                        ac(ieq,idofm)=(ff2*T2-ff1*T1)/xflow360
+                        ac(ieq,idofm)=0.d0
                      endif
-!     
-                     if(idofp1.ne.0) then
-                        ac(ieq,idofp1)=+2.d0*xcst*dt1*xk1
-     &                       /(pt1*xdenom1)
-                     endif
-!     
                      if(idofp2.ne.0) then
-                        ac(ieq,idofp2)=-2.d0*xcst*dt2*xk2
-     &                       /(pt2*xdenom2)
-                    endif
-                 endif
+                        ac(ieq,idofp2)=-ff2*T2/pt2
+                     endif
+                     if(idoft2.ne.0) then
+                        ac(ieq,idoft2)=(ff2/2.d0-1.d0)/(1.d0+bb*M2**2)
+                     endif
+                   else
+                     if(idofp1.ne.0) then
+                        ac(ieq,idofp1)=ff1*T1/pt1
+                     endif
+                     if(idoft1.ne.0) then
+                        ac(ieq,idoft1)=(1.d0-ff1/2.d0)/(1.d0+bb*M1**2)
+                     endif
+!
+!                 removal of the dependency of the energy equation
+!                 on the mass seems to be beneficial for convergence
 !     
+                     if(idofm.ne.0) then
+c                        ac(ieq,idofm)=-ff1*T1/xflow360
+                        ac(ieq,idofm)=0.d0
+                     endif
+                     if(idoft2.ne.0) then
+                        ac(ieq,idoft2)=-1.d0/(1.d0+bb/kappa)
+                     endif
+                  endif
+!
               endif 
            endif
 !     
@@ -418,7 +575,8 @@
 !     
 !     for liquids: determine the gravity vector
 !     
-           if(lakon(nelem)(2:3).eq.'LI') then
+           if((lakon(nelem)(2:3).eq.'LI').or.
+     &        (lakon(nelem)(2:3).eq.'LP')) then
               do j=1,3
                  g(j)=0.d0
                enddo
@@ -441,7 +599,8 @@
             call flux(node1,node2,nodem,nelem,lakon,kon,ipkon,
      &           nactdog,identity,ielprop,prop,kflag,v,xflow,f,
      &           nodef,idirf,df,cp,R,rho,physcon,g,co,dvi,numf,
-     &           vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider)
+     &           vold,set,shcon,nshcon,rhcon,nrhcon,ntmat_,mi,ider,
+     &           ttime,time,iaxial,iplausi)
 !
             do k=1,numf
                idof=nactdog(idirf(k),nodef(k))
@@ -488,8 +647,10 @@
                nopes=3
             elseif(lakonl(4:5).eq.'15') then
                nope=15
-            else
+            elseif(lakonl(4:4).eq.'6') then
                nope=6
+            else
+               cycle
             endif
 !     
             if(lakonl(4:5).eq.'8R') then
@@ -529,6 +690,12 @@
 !     
 !     connectivity of the element
 !     
+            index=ipkon(nelem)
+            if(index.lt.0) then
+               write(*,*) '*ERROR in mafillnet: element ',nelem
+               write(*,*) '       is not defined'
+               call exit(201)
+            endif
             do k=1,nope
                konl(k)=kon(index+k)
             enddo
@@ -625,9 +792,12 @@
                   sinktemp=v(0,node)
                   call film(h,sinktemp,temp,istep,
      &                 iinc,tvar,nelem,l,coords,jltyp,field,nfield,
-     &                 sideload(i),node,areaj,v,mi)
-                  if(nmethod.eq.1) h(1)=xloadold(1,i)+
-     &                 (h(1)-xloadold(1,i))*reltime
+     &                 sideload(i),node,areaj,v,mi,ipkon,kon,lakon,
+     &                 iponoeln,inoeln,ielprop,prop,ielmat,shcon,nshcon,
+     &                 rhcon,nrhcon,ntmat_,cocon,ncocon,
+     &                 ipobody,xbodyact,ibody,heatnod,heatfac)
+c                  if(nmethod.eq.1) h(1)=xloadold(1,i)+
+c     &                 (h(1)-xloadold(1,i))*reltime
                endif
 !     
                idoft=nactdog(0,node)
@@ -648,23 +818,34 @@
       do i=nmpc,1,-1
          if(labmpc(i)(1:7).ne.'NETWORK') cycle
          j=j-1
-         index=ipompc(i)
+         if(labmpc(i)(8:8).eq.' ') then
+            index=ipompc(i)
 !
-         do
-            node=nodempc(1,index)
-            idir=nodempc(2,index)
-            if(nactdog(idir,node).ne.0) then
-               ac(j,nactdog(idir,node))=coefmpc(index)
-            endif
-            index=nodempc(3,index)
-            if(index.eq.0) exit
-         enddo
+!           linear equation
+!
+            do
+               node=nodempc(1,index)
+               idir=nodempc(2,index)
+               if(nactdog(idir,node).ne.0) then
+                  ac(j,nactdog(idir,node))=coefmpc(index)
+               endif
+               index=nodempc(3,index)
+               if(index.eq.0) exit
+            enddo
+         else
+!
+!           user-defined network equation
+!
+            call networkmpc_lhs(i,ipompc,nodempc,coefmpc,labmpc,
+     &          v,nactdog,ac,j,mi,nteq,ipkon,kon,lakon,iponoeln,
+     &          inoeln,ielprop,prop,ielmat,
+     &          shcon,nshcon,rhcon,nrhcon,ntmat_,cocon,ncocon)
+         endif
       enddo
 !
-c      write(*,*) 'ac in mafillgas'
-c      write(*,*) nteq
-c      do i=1,51
-c         write(*,'(17(1x,e11.4))') (ac(i,j),j=1,51)
+c      write(*,*) nteq,' mafillnet '
+c      do i=1,nteq
+c         write(*,'(17(1x,e11.4))') (ac(i,j),j=1,nteq)
 c      enddo
 !
       return
