@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!     Copyright (C) 1998-2011 Guido Dhondt
+!     Copyright (C) 1998-2025 Guido Dhondt
 !     
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,11 @@
       subroutine rcavi_cp_lt(xflow)
 !     
 !     rcavi element
+!
+!     author: Yannick Muller
 ! 
+      implicit none
+!
       real*8 xflow
 !
       write(*,*)'xflow',xflow
