@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -16,9 +16,9 @@
 !     along with this program; if not, write to the Free Software
 !     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 !
-      subroutine rubber(elconloc,elas,emec,kode,didc,
+      subroutine rubber(elconloc,stiff,emec,kode,didc,
      &  d2idc2,dibdc,d2ibdc2,dudc,d2udc2,dldc,d2ldc2,dlbdc,d2lbdc2,
-     &  ithermal,icmd,beta,stre)
+     &  ithermal,icmd,beta,stre,ncmat_)
 !
 !     calculates stiffness and stresses for rubber and elastomeric
 !     foam materials
@@ -31,9 +31,9 @@
       integer ogden,hyperfoam,taylor
 !
       integer nelconst,kode,kk(84),i,j,k,l,m,nt,icmd,istart,iend,
-     &  nc,n,ithermal,ii,jj,mm,neig
+     &  nc,n,ithermal(*),ii,jj,mm,neig,ncmat_
 !
-      real*8 elconloc(*),elas(*),emec(*),didc(3,3,3),
+      real*8 elconloc(*),stiff(*),emec(*),didc(3,3,3),
      &  d2idc2(3,3,3,3,3),dibdc(3,3,3),d2ibdc2(3,3,3,3,3),dudc(3,3),
      &  d2udc2(3,3,3,3),dldc(3,3,3),d2ldc2(3,3,3,3,3),dlbdc(3,3,3),
      &  d2lbdc2(3,3,3,3,3),v1,v2,v3,c(3,3),cinv(3,3),d(3,3),djth,
@@ -50,7 +50,7 @@
 !     they can be mixed without influencing the field in the
 !     calling program
 !
-      do i=1,21
+      do i=1,ncmat_
          constant(i)=elconloc(i)
       enddo
 !
@@ -412,7 +412,7 @@ c      if(icmd.eq.1) then
                         d2ldc2(k,l,m,n,i)=(dcos(tt+(i-1)*2.d0*pi/3.d0)*
      &                   (-(2.d0*v1*didc(k,l,1)-3.d0*didc(k,l,2))*
      &                    (2.d0*v1*didc(m,n,1)-3.d0*didc(m,n,2))/
-     &                    (6.d0*(v1*v1-3.d0*v2)**1.5)+
+     &                    (6.d0*(v1*v1-3.d0*v2)**1.5d0)+
      &                    (2.d0*didc(k,l,1)*didc(m,n,1)+2.d0*v1*
      &                     d2idc2(k,l,m,n,1)-3.d0*d2idc2(k,l,m,n,2))/
      &                     (3.d0*dsqrt(v1*v1-3.d0*v2)))
@@ -828,7 +828,7 @@ c      if(icmd.eq.1) then
                m=kk(nt+3)
                n=kk(nt+4)
                nt=nt+4
-               elas(i)=4.d0*d2udc2(k,l,m,n)
+               stiff(i)=4.d0*d2udc2(k,l,m,n)
             enddo
          endif
 !
