@@ -1,5 +1,5 @@
 /*     CalculiX - A 3-dimensional finite element program                 */
-/*              Copyright (C) 1998-2011 Guido Dhondt                          */
+/*              Copyright (C) 1998-2025 Guido Dhondt                          */
 
 /*     This program is free software; you can redistribute it and/or     */
 /*     modify it under the terms of the GNU General Public License as    */
@@ -21,13 +21,13 @@
 #include <string.h>
 #include "CalculiX.h"
 
-int strcmp2(const char *s1, const char *s2, int length)
+ITG strcmp2(const char *s1, const char *s2, ITG length)
 {
 
 /* comparison of the first "length" characters unless s1
    and/or s2 has less characters */
 
-  int a,b,i;
+  ITG a,b,i;
 
   i=0;
   do {
