@@ -1,6 +1,6 @@
 !
 !     CalculiX - A 3-dimensional finite element program
-!              Copyright (C) 1998-2011 Guido Dhondt
+!              Copyright (C) 1998-2025 Guido Dhondt
 !
 !     This program is free software; you can redistribute it and/or
 !     modify it under the terms of the GNU General Public License as
@@ -18,7 +18,7 @@
 !
       subroutine writevector(ad,neq,number)
 !
-!     writes an MPC to standard output (for debugging purposes)
+!      writes an vector to file (for debugging purposes)
 !
       implicit none
 !      
@@ -35,9 +35,9 @@
       write(10,*) 'vector  number ',number
 !      
       do i=1,neq
-         if(ad(i).gt.1.0e-10.or.ad(i).lt.-1.0e-10)then
+c         if(ad(i).gt.1.0e-10.or.ad(i).lt.-1.0e-10)then
             write(10,*) 'row ',i,' value ',ad(i)
-         endif
+c         endif
       enddo
 !
       close(10)
